#ifndef __DMAC_H_
#define __DMAC_H_
// DMAC: CHANNEL REGISTERS structs


//Dn_CHCR
typedef struct {
	 u_int DIR :1;	// direction
	 u_int pad0:1;	// padding
	 u_int MOD :2;	// mode
	 u_int ASP :2;	// address stack pointer
	 u_int TTE :1;	// tag transfer enable
	 u_int TIE :1;	// tag interrupt enable
	 u_int STR :1;	// start
	 u_int pad1:7;	// padding
	 u_int TAG :16;	// dmatag
} stDn_CHCR;
// Dn_CHCR SET macro
#define LIB_SET_D0_CHCR	LIB_SET_Dn_CHCR
#define LIB_SET_D1_CHCR	LIB_SET_Dn_CHCR
#define LIB_SET_D2_CHCR	LIB_SET_Dn_CHCR
#define LIB_SET_D3_CHCR	LIB_SET_Dn_CHCR
#define LIB_SET_D4_CHCR	LIB_SET_Dn_CHCR
#define LIB_SET_D5_CHCR	LIB_SET_Dn_CHCR
#define LIB_SET_D6_CHCR	LIB_SET_Dn_CHCR
#define LIB_SET_D7_CHCR	LIB_SET_Dn_CHCR
#define LIB_SET_D8_CHCR	LIB_SET_Dn_CHCR
#define LIB_SET_D9_CHCR	LIB_SET_Dn_CHCR
#define LIB_SET_Dn_CHCR(DIR, MOD, ASP, TTE, TIE, STR, TAG) \
	(((u_long)(DIR) << 0)  | ((u_long)(MOD) << 2)  |\
	 ((u_long)(ASP) << 4)  | ((u_long)(TTE) << 6)  |\
	 ((u_long)(TIE) << 7)  | ((u_long)(STR) << 8)  |\
	 ((u_long)(TAG) << 16))
#define Dn_CHCR_DIR_M	(0x1<<0)
#define Dn_CHCR_MOD_M	(0x3<<2)
#define Dn_CHCR_ASP_M	(0x3<<4)
#define Dn_CHCR_TTE_M	(0x1<<6)
#define Dn_CHCR_TIE_M	(0x1<<7)
#define Dn_CHCR_STR_M	(0x1<<8)
#define Dn_CHCR_TAG_M	(0xFFFF<<16)

#define Dn_CHCR_DIR_O	(0)
#define Dn_CHCR_MOD_O	(2)
#define Dn_CHCR_ASP_O	(4)
#define Dn_CHCR_TTE_O	(6)
#define Dn_CHCR_TIE_O	(7)
#define Dn_CHCR_STR_O	(8)
#define Dn_CHCR_TAG_O	(16)
// Dn_CHCR instances
#define D0_CHCR 	((volatile u_int *)(0x10008000))
#define D1_CHCR 	((volatile u_int *)(0x10009000))
#define D2_CHCR 	((volatile u_int *)(0x1000a000))
#define D3_CHCR 	((volatile u_int *)(0x1000b000))
#define D4_CHCR 	((volatile u_int *)(0x1000b400))
#define D5_CHCR 	((volatile u_int *)(0x1000c000))
#define D6_CHCR 	((volatile u_int *)(0x1000c400))
#define D7_CHCR 	((volatile u_int *)(0x1000c800))
#define D8_CHCR 	((volatile u_int *)(0x1000d000))
#define D9_CHCR 	((volatile u_int *)(0x1000d400))



//Dn_MADR
typedef struct {
	 u_int ADDR:31;	// memory address
	 u_int SPR :1;	// memory/spr selection
} stDn_MADR;
// Dn_MADR SET macro
#define LIB_SET_D0_MADR	LIB_SET_Dn_MADR
#define LIB_SET_D1_MADR	LIB_SET_Dn_MADR
#define LIB_SET_D2_MADR	LIB_SET_Dn_MADR
#define LIB_SET_D3_MADR	LIB_SET_Dn_MADR
#define LIB_SET_D4_MADR	LIB_SET_Dn_MADR
#define LIB_SET_D5_MADR	LIB_SET_Dn_MADR
#define LIB_SET_D6_MADR	LIB_SET_Dn_MADR
#define LIB_SET_D7_MADR	LIB_SET_Dn_MADR
#define LIB_SET_D8_MADR	LIB_SET_Dn_MADR
#define LIB_SET_D9_MADR	LIB_SET_Dn_MADR
#define LIB_SET_Dn_MADR(ADDR, SPR) \
	(((u_long)(ADDR) << 0)   | ((u_long)(SPR) << 31))
#define Dn_MADR_ADDR_M	(0x7FFFFFFF<<0)
#define Dn_MADR_SPR_M	(0x1<<31)

#define Dn_MADR_ADDR_O	(0)
#define Dn_MADR_SPR_O	(31)
// Dn_MADR instances
#define D0_MADR 	((volatile u_int *)(0x10008010))
#define D1_MADR 	((volatile u_int *)(0x10009010))
#define D2_MADR 	((volatile u_int *)(0x1000a010))
#define D3_MADR 	((volatile u_int *)(0x1000b010))
#define D4_MADR 	((volatile u_int *)(0x1000b410))
#define D5_MADR 	((volatile u_int *)(0x1000c010))
#define D6_MADR 	((volatile u_int *)(0x1000c410))
#define D7_MADR 	((volatile u_int *)(0x1000c810))
#define D8_MADR 	((volatile u_int *)(0x1000d010))
#define D9_MADR 	((volatile u_int *)(0x1000d410))



//Dn_TADR
typedef struct {
	 u_int ADDR:31;	// next tag address
	 u_int SPR :1;	// next memory/spr selection
} stDn_TADR;
// Dn_TADR SET macro
#define LIB_SET_D0_TADR	LIB_SET_Dn_TADR
#define LIB_SET_D1_TADR	LIB_SET_Dn_TADR
#define LIB_SET_D2_TADR	LIB_SET_Dn_TADR
#define LIB_SET_D4_TADR	LIB_SET_Dn_TADR
#define LIB_SET_D6_TADR	LIB_SET_Dn_TADR
#define LIB_SET_D9_TADR	LIB_SET_Dn_TADR
#define LIB_SET_Dn_TADR(ADDR, SPR) \
	(((u_long)(ADDR) << 0)   | ((u_long)(SPR) << 31))
#define Dn_TADR_ADDR_M	(0x7FFFFFFF<<0)
#define Dn_TADR_SPR_M	(0x1<<31)

#define Dn_TADR_ADDR_O	(0)
#define Dn_TADR_SPR_O	(31)
// Dn_TADR instances
#define D0_TADR 	((volatile u_int *)(0x10008030))
#define D1_TADR 	((volatile u_int *)(0x10009030))
#define D2_TADR 	((volatile u_int *)(0x1000a030))
#define D4_TADR 	((volatile u_int *)(0x1000b430))
#define D6_TADR 	((volatile u_int *)(0x1000c430))
#define D9_TADR 	((volatile u_int *)(0x1000d430))



//Dn_ASRx
typedef struct {
	 u_int ADDR:31;	// tag memory address
	 u_int SPR :1;	// memory/spr selection
} stDn_ASRx;
// Dn_ASRx SET macro
#define LIB_SET_D0_ASR0	LIB_SET_Dn_ASRx
#define LIB_SET_D0_ASR1	LIB_SET_Dn_ASRx
#define LIB_SET_D1_ASR0	LIB_SET_Dn_ASRx
#define LIB_SET_D1_ASR1	LIB_SET_Dn_ASRx
#define LIB_SET_D2_ASR0	LIB_SET_Dn_ASRx
#define LIB_SET_D2_ASR1	LIB_SET_Dn_ASRx
#define LIB_SET_Dn_ASRx(ADDR, SPR) \
	(((u_long)(ADDR) << 0)   | ((u_long)(SPR) << 31))
#define Dn_ASRx_ADDR_M	(0x7FFFFFFF<<0)
#define Dn_ASRx_SPR_M	(0x1<<31)

#define Dn_ASRx_ADDR_O	(0)
#define Dn_ASRx_SPR_O	(31)
// Dn_ASRx instances
#define D0_ASR0 	((volatile u_int *)(0x10008040))
#define D0_ASR1 	((volatile u_int *)(0x10008050))
#define D1_ASR0 	((volatile u_int *)(0x10009040))
#define D1_ASR1 	((volatile u_int *)(0x10009050))
#define D2_ASR0 	((volatile u_int *)(0x1000a040))
#define D2_ASR1 	((volatile u_int *)(0x1000a050))



//Dn_SADR
typedef struct {
	 u_int ADDR:14;	// spr address
	 u_int pad0:18;	// padding
} stDn_SADR;
// Dn_SADR SET macro
#define LIB_SET_D8_SADR	LIB_SET_Dn_SADR
#define LIB_SET_D9_SADR	LIB_SET_Dn_SADR
#define LIB_SET_Dn_SADR(ADDR) \
	(((u_long)(ADDR) << 0))
#define Dn_SADR_ADDR_M	(0x3FFF<<0)

#define Dn_SADR_ADDR_O	(0)
// Dn_SADR instances
#define D8_SADR 	((volatile u_int *)(0x1000d080))
#define D9_SADR 	((volatile u_int *)(0x1000d480))



//Dn_QWC
typedef struct {
	 u_int QWC :16;	// quadword counter, transfer size
	 u_int pad0:16;	// padding
} stDn_QWC;
// Dn_QWC SET macro
#define LIB_SET_D0_QWC	LIB_SET_Dn_QWC
#define LIB_SET_D1_QWC	LIB_SET_Dn_QWC
#define LIB_SET_D2_QWC	LIB_SET_Dn_QWC
#define LIB_SET_D3_QWC	LIB_SET_Dn_QWC
#define LIB_SET_D4_QWC	LIB_SET_Dn_QWC
#define LIB_SET_D5_QWC	LIB_SET_Dn_QWC
#define LIB_SET_D6_QWC	LIB_SET_Dn_QWC
#define LIB_SET_D7_QWC	LIB_SET_Dn_QWC
#define LIB_SET_D8_QWC	LIB_SET_Dn_QWC
#define LIB_SET_D9_QWC	LIB_SET_Dn_QWC
#define LIB_SET_Dn_QWC(QWC) \
	(((u_long)(QWC) << 0))
#define Dn_QWC_QWC_M	(0xFFFF<<0)

#define Dn_QWC_QWC_O	(0)
// Dn_QWC instances
#define D0_QWC 	((volatile u_int *)(0x10008020))
#define D1_QWC 	((volatile u_int *)(0x10009020))
#define D2_QWC 	((volatile u_int *)(0x1000a020))
#define D3_QWC 	((volatile u_int *)(0x1000b020))
#define D4_QWC 	((volatile u_int *)(0x1000b420))
#define D5_QWC 	((volatile u_int *)(0x1000c020))
#define D6_QWC 	((volatile u_int *)(0x1000c420))
#define D7_QWC 	((volatile u_int *)(0x1000c820))
#define D8_QWC 	((volatile u_int *)(0x1000d020))
#define D9_QWC 	((volatile u_int *)(0x1000d420))


#endif
//
