#ifndef __DMACR_H_
#define __DMACR_H_
// DMAC structs


//D_CTRL
typedef struct {
	 u_int DMAE:1;	// dma enable
	 u_int RELE:1;	// release signal enable
	 u_int MFD :2;	// memory fifo drain channel
	 u_int STS :2;	// stall control source channel
	 u_int STD :2;	// stall control drain channel
	 u_int RCYC:3;	// release cycle
	 u_int pad0:21;	// padding
} stD_CTRL;
// D_CTRL SET macro
#define LIB_SET_D_CTRL(DMAE, RELE, MFD, STS, STD, RCYC) \
	(((u_long)(DMAE) << 0)  | ((u_long)(RELE) << 1)  |\
	 ((u_long)(MFD) << 2)   | ((u_long)(STS) << 4)   |\
	 ((u_long)(STD) << 6)   | ((u_long)(RCYC) << 8))
#define D_CTRL_DMAE_M	(0x1<<0)
#define D_CTRL_RELE_M	(0x1<<1)
#define D_CTRL_MFD_M	(0x3<<2)
#define D_CTRL_STS_M	(0x3<<4)
#define D_CTRL_STD_M	(0x3<<6)
#define D_CTRL_RCYC_M	(0x7<<8)

#define D_CTRL_DMAE_O	(0)
#define D_CTRL_RELE_O	(1)
#define D_CTRL_MFD_O	(2)
#define D_CTRL_STS_O	(4)
#define D_CTRL_STD_O	(6)
#define D_CTRL_RCYC_O	(8)
// D_CTRL instances
#define D_CTRL 	((volatile u_int *)(0x1000e000))



//D_STAT
typedef struct {
	 u_int CIS0:1;	// channel interrupt status 0
	 u_int CIS1:1;	// channel interrupt status 1
	 u_int CIS2:1;	// channel interrupt status 2
	 u_int CIS3:1;	// channel interrupt status 3
	 u_int CIS4:1;	// channel interrupt status 4
	 u_int CIS5:1;	// channel interrupt status 5
	 u_int CIS6:1;	// channel interrupt status 6
	 u_int CIS7:1;	// channel interrupt status 7
	 u_int CIS8:1;	// channel interrupt status 8
	 u_int CIS9:1;	// channel interrupt status 9
	 u_int pad0:3;	// padding
	 u_int SIS :1;	// dma stall interrupt status
	 u_int MEIS:1;	// mfifo empty interrupt status
	 u_int BEIS:1;	// buserr interrupt status
	 u_int CIM0:1;	// channel interrupt mask 0
	 u_int CIM1:1;	// channel interrupt mask 1
	 u_int CIM2:1;	// channel interrupt mask 2
	 u_int CIM3:1;	// channel interrupt mask 3
	 u_int CIM4:1;	// channel interrupt mask 4
	 u_int CIM5:1;	// channel interrupt mask 5
	 u_int CIM6:1;	// channel interrupt mask 6
	 u_int CIM7:1;	// channel interrupt mask 7
	 u_int CIM8:1;	// channel interrupt mask 8
	 u_int CIM9:1;	// channel interrupt mask 9
	 u_int pad1:3;	// padding
	 u_int SIM :1;	// dma stall interrupt mask
	 u_int MEIM:1;	// mfifo empty interrupt mask
	 u_int pad2:1;	// padding
} stD_STAT;
// D_STAT SET macro
#define LIB_SET_D_STAT(CIS0, CIS1, CIS2, CIS3, CIS4, CIS5, CIS6, CIS7, CIS8, CIS9, SIS, MEIS, BEIS, CIM0, CIM1, CIM2, CIM3, CIM4, CIM5, CIM6, CIM7, CIM8, CIM9, SIM, MEIM) \
	(((u_long)(CIS0) << 0)    | ((u_long)(CIS1) << 1)    |\
	 ((u_long)(CIS2) << 2)    | ((u_long)(CIS3) << 3)    |\
	 ((u_long)(CIS4) << 4)    | ((u_long)(CIS5) << 5)    |\
	 ((u_long)(CIS6) << 6)    | ((u_long)(CIS7) << 7)    |\
	 ((u_long)(CIS8) << 8)    | ((u_long)(CIS9) << 9)    |\
	 ((u_long)(SIS) << 13)    | ((u_long)(MEIS) << 14)   |\
	 ((u_long)(BEIS) << 15)   | ((u_long)(CIM0) << 16)   |\
	 ((u_long)(CIM1) << 17)   | ((u_long)(CIM2) << 18)   |\
	 ((u_long)(CIM3) << 19)   | ((u_long)(CIM4) << 20)   |\
	 ((u_long)(CIM5) << 21)   | ((u_long)(CIM6) << 22)   |\
	 ((u_long)(CIM7) << 23)   | ((u_long)(CIM8) << 24)   |\
	 ((u_long)(CIM9) << 25)   | ((u_long)(SIM) << 29)    |\
	 ((u_long)(MEIM) << 30))
#define D_STAT_CIS0_M	(0x1<<0)
#define D_STAT_CIS1_M	(0x1<<1)
#define D_STAT_CIS2_M	(0x1<<2)
#define D_STAT_CIS3_M	(0x1<<3)
#define D_STAT_CIS4_M	(0x1<<4)
#define D_STAT_CIS5_M	(0x1<<5)
#define D_STAT_CIS6_M	(0x1<<6)
#define D_STAT_CIS7_M	(0x1<<7)
#define D_STAT_CIS8_M	(0x1<<8)
#define D_STAT_CIS9_M	(0x1<<9)
#define D_STAT_SIS_M	(0x1<<13)
#define D_STAT_MEIS_M	(0x1<<14)
#define D_STAT_BEIS_M	(0x1<<15)
#define D_STAT_CIM0_M	(0x1<<16)
#define D_STAT_CIM1_M	(0x1<<17)
#define D_STAT_CIM2_M	(0x1<<18)
#define D_STAT_CIM3_M	(0x1<<19)
#define D_STAT_CIM4_M	(0x1<<20)
#define D_STAT_CIM5_M	(0x1<<21)
#define D_STAT_CIM6_M	(0x1<<22)
#define D_STAT_CIM7_M	(0x1<<23)
#define D_STAT_CIM8_M	(0x1<<24)
#define D_STAT_CIM9_M	(0x1<<25)
#define D_STAT_SIM_M	(0x1<<29)
#define D_STAT_MEIM_M	(0x1<<30)

#define D_STAT_CIS0_O	(0)
#define D_STAT_CIS1_O	(1)
#define D_STAT_CIS2_O	(2)
#define D_STAT_CIS3_O	(3)
#define D_STAT_CIS4_O	(4)
#define D_STAT_CIS5_O	(5)
#define D_STAT_CIS6_O	(6)
#define D_STAT_CIS7_O	(7)
#define D_STAT_CIS8_O	(8)
#define D_STAT_CIS9_O	(9)
#define D_STAT_SIS_O	(13)
#define D_STAT_MEIS_O	(14)
#define D_STAT_BEIS_O	(15)
#define D_STAT_CIM0_O	(16)
#define D_STAT_CIM1_O	(17)
#define D_STAT_CIM2_O	(18)
#define D_STAT_CIM3_O	(19)
#define D_STAT_CIM4_O	(20)
#define D_STAT_CIM5_O	(21)
#define D_STAT_CIM6_O	(22)
#define D_STAT_CIM7_O	(23)
#define D_STAT_CIM8_O	(24)
#define D_STAT_CIM9_O	(25)
#define D_STAT_SIM_O	(29)
#define D_STAT_MEIM_O	(30)
// D_STAT instances
#define D_STAT 	((volatile u_int *)(0x1000e010))



//D_PCR
typedef struct {
	 u_int CPC0:1;	// cop control 0
	 u_int CPC1:1;	// cop control 1
	 u_int CPC2:1;	// cop control 2
	 u_int CPC3:1;	// cop control 3
	 u_int CPC4:1;	// cop control 4
	 u_int CPC5:1;	// cop control 5
	 u_int CPC6:1;	// cop control 6
	 u_int CPC7:1;	// cop control 7
	 u_int CPC8:1;	// cop control 8
	 u_int CPC9:1;	// cop control 9
	 u_int pad0:6;	// padding
	 u_int CDE0:1;	// channel dma enable 0
	 u_int CDE1:1;	// channel dma enable 1
	 u_int CDE2:1;	// channel dma enable 2
	 u_int CDE3:1;	// channel dma enable 3
	 u_int CDE4:1;	// channel dma enable 4
	 u_int CDE5:1;	// channel dma enable 5
	 u_int CDE6:1;	// channel dma enable 6
	 u_int CDE7:1;	// channel dma enable 7
	 u_int CDE8:1;	// channel dma enable 8
	 u_int CDE9:1;	// channel dma enable 9
	 u_int pad1:5;	// padding
	 u_int PCE :1;	// priority control enable
} stD_PCR;
// D_PCR SET macro
#define LIB_SET_D_PCR(CPC0, CPC1, CPC2, CPC3, CPC4, CPC5, CPC6, CPC7, CPC8, CPC9, CDE0, CDE1, CDE2, CDE3, CDE4, CDE5, CDE6, CDE7, CDE8, CDE9, PCE) \
	(((u_long)(CPC0) << 0)   | ((u_long)(CPC1) << 1)   |\
	 ((u_long)(CPC2) << 2)   | ((u_long)(CPC3) << 3)   |\
	 ((u_long)(CPC4) << 4)   | ((u_long)(CPC5) << 5)   |\
	 ((u_long)(CPC6) << 6)   | ((u_long)(CPC7) << 7)   |\
	 ((u_long)(CPC8) << 8)   | ((u_long)(CPC9) << 9)   |\
	 ((u_long)(CDE0) << 16)  | ((u_long)(CDE1) << 17)  |\
	 ((u_long)(CDE2) << 18)  | ((u_long)(CDE3) << 19)  |\
	 ((u_long)(CDE4) << 20)  | ((u_long)(CDE5) << 21)  |\
	 ((u_long)(CDE6) << 22)  | ((u_long)(CDE7) << 23)  |\
	 ((u_long)(CDE8) << 24)  | ((u_long)(CDE9) << 25)  |\
	 ((u_long)(PCE) << 31))
#define D_PCR_CPC0_M	(0x1<<0)
#define D_PCR_CPC1_M	(0x1<<1)
#define D_PCR_CPC2_M	(0x1<<2)
#define D_PCR_CPC3_M	(0x1<<3)
#define D_PCR_CPC4_M	(0x1<<4)
#define D_PCR_CPC5_M	(0x1<<5)
#define D_PCR_CPC6_M	(0x1<<6)
#define D_PCR_CPC7_M	(0x1<<7)
#define D_PCR_CPC8_M	(0x1<<8)
#define D_PCR_CPC9_M	(0x1<<9)
#define D_PCR_CDE0_M	(0x1<<16)
#define D_PCR_CDE1_M	(0x1<<17)
#define D_PCR_CDE2_M	(0x1<<18)
#define D_PCR_CDE3_M	(0x1<<19)
#define D_PCR_CDE4_M	(0x1<<20)
#define D_PCR_CDE5_M	(0x1<<21)
#define D_PCR_CDE6_M	(0x1<<22)
#define D_PCR_CDE7_M	(0x1<<23)
#define D_PCR_CDE8_M	(0x1<<24)
#define D_PCR_CDE9_M	(0x1<<25)
#define D_PCR_PCE_M	(0x1<<31)

#define D_PCR_CPC0_O	(0)
#define D_PCR_CPC1_O	(1)
#define D_PCR_CPC2_O	(2)
#define D_PCR_CPC3_O	(3)
#define D_PCR_CPC4_O	(4)
#define D_PCR_CPC5_O	(5)
#define D_PCR_CPC6_O	(6)
#define D_PCR_CPC7_O	(7)
#define D_PCR_CPC8_O	(8)
#define D_PCR_CPC9_O	(9)
#define D_PCR_CDE0_O	(16)
#define D_PCR_CDE1_O	(17)
#define D_PCR_CDE2_O	(18)
#define D_PCR_CDE3_O	(19)
#define D_PCR_CDE4_O	(20)
#define D_PCR_CDE5_O	(21)
#define D_PCR_CDE6_O	(22)
#define D_PCR_CDE7_O	(23)
#define D_PCR_CDE8_O	(24)
#define D_PCR_CDE9_O	(25)
#define D_PCR_PCE_O	(31)
// D_PCR instances
#define D_PCR 	((volatile u_int *)(0x1000e020))



//D_SQWC
typedef struct {
	 u_int SQWC:8;	// skip quadword counter
	 u_int pad0:8;	// padding
	 u_int TQWC:8;	// transfer quadword counter
	 u_int pad1:8;	// padding
} stD_SQWC;
// D_SQWC SET macro
#define LIB_SET_D_SQWC(SQWC, TQWC) \
	(((u_long)(SQWC) << 0)   | ((u_long)(TQWC) << 16))
#define D_SQWC_SQWC_M	(0xFF<<0)
#define D_SQWC_TQWC_M	(0xFF<<16)

#define D_SQWC_SQWC_O	(0)
#define D_SQWC_TQWC_O	(16)
// D_SQWC instances
#define D_SQWC 	((volatile u_int *)(0x1000e030))



//D_RBOR
typedef struct {
	 u_int ADDR:31;	// ring buffer offset address
	 u_int pad0:1;	// padding
} stD_RBOR;
// D_RBOR SET macro
#define LIB_SET_D_RBOR(ADDR) \
	(((u_long)(ADDR) << 0))
#define D_RBOR_ADDR_M	(0x7FFFFFFF<<0)

#define D_RBOR_ADDR_O	(0)
// D_RBOR instances
#define D_RBOR 	((volatile u_int *)(0x1000e050))



//D_RBSR
typedef struct {
	 u_int pad0:4;	// padding
	 u_int RMSK:27;	// ring buffer size mask
	 u_int pad1:1;	// padding
} stD_RBSR;
// D_RBSR SET macro
#define LIB_SET_D_RBSR(RMSK) \
	(((u_long)(RMSK) << 4))
#define D_RBSR_RMSK_M	(0x7FFFFFF<<4)

#define D_RBSR_RMSK_O	(4)
// D_RBSR instances
#define D_RBSR 	((volatile u_int *)(0x1000e040))



//D_STADR
typedef struct {
	 u_int ADDR:31;	// stall address
	 u_int pad0:1;	// padding
} stD_STADR;
// D_STADR SET macro
#define LIB_SET_D_STADR(ADDR) \
	(((u_long)(ADDR) << 0))
#define D_STADR_ADDR_M	(0x7FFFFFFF<<0)

#define D_STADR_ADDR_O	(0)
// D_STADR instances
#define D_STADR 	((volatile u_int *)(0x1000e060))



//D_ENABLER
typedef struct {
	 u_int pad0:16;	// padding
	 u_int CPND:1;	// dma transfer hold state
	 u_int pad1:15;	// padding
} stD_ENABLER;
// D_ENABLER SET macro
#define LIB_SET_D_ENABLER(CPND) \
	(((u_long)(CPND) << 16))
#define D_ENABLER_CPND_M	(0x1<<16)

#define D_ENABLER_CPND_O	(16)
// D_ENABLER instances
#define D_ENABLER 	((volatile u_int *)(0x1000f520))



//D_ENABLEW
typedef struct {
	 u_int pad0:16;	// padding
	 u_int CPND:1;	// dma transfer is held
	 u_int pad1:15;	// padding
} stD_ENABLEW;
// D_ENABLEW SET macro
#define LIB_SET_D_ENABLEW(CPND) \
	(((u_long)(CPND) << 16))
#define D_ENABLEW_CPND_M	(0x1<<16)

#define D_ENABLEW_CPND_O	(16)
// D_ENABLEW instances
#define D_ENABLEW 	((volatile u_int *)(0x1000f590))


#endif
//
