#ifndef __GIF_H_
#define __GIF_H_
// GIF CONTROL REGISTER structs


//GIF_CTRL
typedef struct {
	 u_int RST :1;	// gif reset
	 u_int pad0:2;	// padding
	 u_int PSE :1;	// temporary transfer stop
	 u_int pad1:28;	// padding
} stGIF_CTRL;
// GIF_CTRL SET macro
#define LIB_SET_GIF_CTRL(RST, PSE) \
	(((u_long)(RST) << 0)  | ((u_long)(PSE) << 3))
#define GIF_CTRL_RST_M	(0x1<<0)
#define GIF_CTRL_PSE_M	(0x1<<3)

#define GIF_CTRL_RST_O	(0)
#define GIF_CTRL_PSE_O	(3)
// GIF_CTRL instances
#define GIF_CTRL 	((volatile u_int *)(0x10003000))



//GIF_MODE
typedef struct {
	 u_int M3R :1;	// path3 mask
	 u_int pad0:1;	// padding
	 u_int IMT :1;	// path3 transfer mode specification
	 u_int pad1:29;	// padding
} stGIF_MODE;
// GIF_MODE SET macro
#define LIB_SET_GIF_MODE(M3R, IMT) \
	(((u_long)(M3R) << 0)  | ((u_long)(IMT) << 2))
#define GIF_MODE_M3R_M	(0x1<<0)
#define GIF_MODE_IMT_M	(0x1<<2)

#define GIF_MODE_M3R_O	(0)
#define GIF_MODE_IMT_O	(2)
// GIF_MODE instances
#define GIF_MODE 	((volatile u_int *)(0x10003010))



//GIF_STAT
typedef struct {
	 u_int M3R  :1;	// path3 mask status
	 u_int M3P  :1;	// path3 vif mask status
	 u_int IMT  :1;	// path3 imt mask status
	 u_int PS2  :1;	// temporary transfer stop
	 u_int pad0 :1;	// padding
	 u_int IP3  :1;	// interrumpted path3
	 u_int P3Q  :1;	// path3 in queue
	 u_int P2Q  :1;	// path2 in queue
	 u_int P1Q  :1;	// path1 in queue
	 u_int OPH  :1;	// output path
	 u_int APATH:2;	// data path transfering data
	 u_int DIR  :1;	// transfer direction
	 u_int pad1 :11;	// padding
	 u_int FQC  :5;	// effective data count in gif-fifo
	 u_int pad2 :3;	// padding
} stGIF_STAT;
// GIF_STAT SET macro
#define LIB_SET_GIF_STAT(M3R, M3P, IMT, PS2, IP3, P3Q, P2Q, P1Q, OPH, APATH, DIR, FQC) \
	(((u_long)(M3R) << 0)    | ((u_long)(M3P) << 1)    |\
	 ((u_long)(IMT) << 2)    | ((u_long)(PS2) << 3)    |\
	 ((u_long)(IP3) << 5)    | ((u_long)(P3Q) << 6)    |\
	 ((u_long)(P2Q) << 7)    | ((u_long)(P1Q) << 8)    |\
	 ((u_long)(OPH) << 9)    | ((u_long)(APATH) << 10) |\
	 ((u_long)(DIR) << 12)   | ((u_long)(FQC) << 24))
#define GIF_STAT_M3R_M	(0x1<<0)
#define GIF_STAT_M3P_M	(0x1<<1)
#define GIF_STAT_IMT_M	(0x1<<2)
#define GIF_STAT_PS2_M	(0x1<<3)
#define GIF_STAT_IP3_M	(0x1<<5)
#define GIF_STAT_P3Q_M	(0x1<<6)
#define GIF_STAT_P2Q_M	(0x1<<7)
#define GIF_STAT_P1Q_M	(0x1<<8)
#define GIF_STAT_OPH_M	(0x1<<9)
#define GIF_STAT_APATH_M	(0x3<<10)
#define GIF_STAT_DIR_M	(0x1<<12)
#define GIF_STAT_FQC_M	(0x1F<<24)

#define GIF_STAT_M3R_O	(0)
#define GIF_STAT_M3P_O	(1)
#define GIF_STAT_IMT_O	(2)
#define GIF_STAT_PS2_O	(3)
#define GIF_STAT_IP3_O	(5)
#define GIF_STAT_P3Q_O	(6)
#define GIF_STAT_P2Q_O	(7)
#define GIF_STAT_P1Q_O	(8)
#define GIF_STAT_OPH_O	(9)
#define GIF_STAT_APATH_O	(10)
#define GIF_STAT_DIR_O	(12)
#define GIF_STAT_FQC_O	(24)
// GIF_STAT instances
#define GIF_STAT 	((volatile u_int *)(0x10003020))



//GIF_TAG0
typedef struct {
	 u_int NLOOP:15;	// most recently nloop
	 u_int EOP  :1;	// most recently eop
	 u_int TAG  :16;	// tag [31:16]
} stGIF_TAG0;
// GIF_TAG0 SET macro
#define LIB_SET_GIF_TAG0(NLOOP, EOP, TAG) \
	(((u_long)(NLOOP) << 0) | ((u_long)(EOP) << 15)  |\
	 ((u_long)(TAG) << 16))
#define GIF_TAG0_NLOOP_M	(0x7FFF<<0)
#define GIF_TAG0_EOP_M	(0x1<<15)
#define GIF_TAG0_TAG_M	(0xFFFF<<16)

#define GIF_TAG0_NLOOP_O	(0)
#define GIF_TAG0_EOP_O	(15)
#define GIF_TAG0_TAG_O	(16)
// GIF_TAG0 instances
#define GIF_TAG0 	((volatile u_int *)(0x10003040))



//GIF_TAG1
typedef struct {
	 u_int TAG :14;	// tag [45:32]
	 u_int PRE :1;	// most recently pre
	 u_int PRIM:11;	// most recently prim
	 u_int FLG :2;	// most recently flg
	 u_int NREG:4;	// most recently nreg
} stGIF_TAG1;
// GIF_TAG1 SET macro
#define LIB_SET_GIF_TAG1(TAG, PRE, PRIM, FLG, NREG) \
	(((u_long)(TAG) << 0)    | ((u_long)(PRE) << 14)   |\
	 ((u_long)(PRIM) << 15)  | ((u_long)(FLG) << 26)   |\
	 ((u_long)(NREG) << 28))
#define GIF_TAG1_TAG_M	(0x3FFF<<0)
#define GIF_TAG1_PRE_M	(0x1<<14)
#define GIF_TAG1_PRIM_M	(0x7FF<<15)
#define GIF_TAG1_FLG_M	(0x3<<26)
#define GIF_TAG1_NREG_M	(0xF<<28)

#define GIF_TAG1_TAG_O	(0)
#define GIF_TAG1_PRE_O	(14)
#define GIF_TAG1_PRIM_O	(15)
#define GIF_TAG1_FLG_O	(26)
#define GIF_TAG1_NREG_O	(28)
// GIF_TAG1 instances
#define GIF_TAG1 	((volatile u_int *)(0x10003050))



//GIF_TAG2
typedef struct {
	 u_int REGS:32;	// most recently regs [64:95]
} stGIF_TAG2;
// GIF_TAG2 SET macro
#define LIB_SET_GIF_TAG2(REGS) \
	(((u_long)(REGS) << 0))
#define GIF_TAG2_REGS_M	(0xFFFFFFFF<<0)

#define GIF_TAG2_REGS_O	(0)
// GIF_TAG2 instances
#define GIF_TAG2 	((volatile u_int *)(0x10003060))



//GIF_TAG3
typedef struct {
	 u_int REGS:32;	// most recently regs [127:96]
} stGIF_TAG3;
// GIF_TAG3 SET macro
#define LIB_SET_GIF_TAG3(REGS) \
	(((u_long)(REGS) << 0))
#define GIF_TAG3_REGS_M	(0xFFFFFFFF<<0)

#define GIF_TAG3_REGS_O	(0)
// GIF_TAG3 instances
#define GIF_TAG3 	((volatile u_int *)(0x10003070))



//GIF_CNT
typedef struct {
	 u_int LOOPCNT:15;	// value of current loop counter (backward counter from nloop)
	 u_int pad0   :1;	// padding
	 u_int REGCNT :4;	// register descriptor number currently in process
	 u_int VUADDR :10;	// address of vu memory under transfer
	 u_int pad1   :2;	// padding
} stGIF_CNT;
// GIF_CNT SET macro
#define LIB_SET_GIF_CNT(LOOPCNT, REGCNT, VUADDR) \
	(((u_long)(LOOPCNT) << 0)  | ((u_long)(REGCNT) << 16)  |\
	 ((u_long)(VUADDR) << 20))
#define GIF_CNT_LOOPCNT_M	(0x7FFF<<0)
#define GIF_CNT_REGCNT_M	(0xF<<16)
#define GIF_CNT_VUADDR_M	(0x3FF<<20)

#define GIF_CNT_LOOPCNT_O	(0)
#define GIF_CNT_REGCNT_O	(16)
#define GIF_CNT_VUADDR_O	(20)
// GIF_CNT instances
#define GIF_CNT 	((volatile u_int *)(0x10003080))



//GIF_P3CNT
typedef struct {
	 u_int P3CNT:15;	// loopcnt field value of gif_cnt register when transfer via path3 is interrupted
	 u_int pad0 :17;	// padding
} stGIF_P3CNT;
// GIF_P3CNT SET macro
#define LIB_SET_GIF_P3CNT(P3CNT) \
	(((u_long)(P3CNT) << 0))
#define GIF_P3CNT_P3CNT_M	(0x7FFF<<0)

#define GIF_P3CNT_P3CNT_O	(0)
// GIF_P3CNT instances
#define GIF_P3CNT 	((volatile u_int *)(0x10003090))



//GIF_P3TAG
typedef struct {
	 u_int LOOPCNT:15;	// nloop field value of giftag read via path3 at the end
	 u_int EOP    :1;	// eop flag value of giftag read via path3 at the end
	 u_int pad0   :16;	// padding
} stGIF_P3TAG;
// GIF_P3TAG SET macro
#define LIB_SET_GIF_P3TAG(LOOPCNT, EOP) \
	(((u_long)(LOOPCNT) << 0) | ((u_long)(EOP) << 15))
#define GIF_P3TAG_LOOPCNT_M	(0x7FFF<<0)
#define GIF_P3TAG_EOP_M	(0x1<<15)

#define GIF_P3TAG_LOOPCNT_O	(0)
#define GIF_P3TAG_EOP_O	(15)
// GIF_P3TAG instances
#define GIF_P3TAG 	((volatile u_int *)(0x100030a0))


#endif
//
