#ifndef __GS_H_
#define __GS_H_
// GS REGISTERS structs


//ALPHA_x
typedef struct {
	 u_long A   :2;	// 
	 u_long B   :2;	// 
	 u_long C   :2;	// 
	 u_long D   :2;	// 
	 u_long pad0:24;	// padding
	 u_long FIX :8;	// 
	 u_long pad1:24;	// padding
} stALPHA_x;
// ALPHA_x SET macro
#define LIB_SET_ALPHA_1	LIB_SET_ALPHA_x
#define LIB_SET_ALPHA_2	LIB_SET_ALPHA_x
#define LIB_SET_ALPHA_x(A, B, C, D, FIX) \
	(((u_long)(A) << 0)        | ((u_long)(B) << 2)        |\
	 ((u_long)(C) << 4)        | ((u_long)(D) << 6)        |\
	 ((u_long)(FIX) << 32))
#define ALPHA_x_A_M	(0x3<<0)
#define ALPHA_x_B_M	(0x3<<2)
#define ALPHA_x_C_M	(0x3<<4)
#define ALPHA_x_D_M	(0x3<<6)
#define ALPHA_x_FIX_M	(0xFF<<32)

#define ALPHA_x_A_O	(0)
#define ALPHA_x_B_O	(2)
#define ALPHA_x_C_O	(4)
#define ALPHA_x_D_O	(6)
#define ALPHA_x_FIX_O	(32)
// ALPHA_x instances



//BITBLTBUF
typedef struct {
	 u_long SBP :14;	// 
	 u_long pad0:2;	// padding
	 u_long SBW :6;	// 
	 u_long pad1:2;	// padding
	 u_long SPSM:6;	// 
	 u_long pad2:2;	// padding
	 u_long DBP :14;	// 
	 u_long pad3:2;	// padding
	 u_long DBW :6;	// 
	 u_long pad4:2;	// padding
	 u_long DPSM:6;	// 
	 u_long pad5:2;	// padding
} stBITBLTBUF;
// BITBLTBUF SET macro
#define LIB_SET_BITBLTBUF(SBP, SBW, SPSM, DBP, DBW, DPSM) \
	(((u_long)(SBP) << 0)        | ((u_long)(SBW) << 16)       |\
	 ((u_long)(SPSM) << 24)      | ((u_long)(DBP) << 32)       |\
	 ((u_long)(DBW) << 48)       | ((u_long)(DPSM) << 56))
#define BITBLTBUF_SBP_M	(0x3FFF<<0)
#define BITBLTBUF_SBW_M	(0x3F<<16)
#define BITBLTBUF_SPSM_M	(0x3F<<24)
#define BITBLTBUF_DBP_M	(0x3FFF<<32)
#define BITBLTBUF_DBW_M	(0x3F<<48)
#define BITBLTBUF_DPSM_M	(0x3F<<56)

#define BITBLTBUF_SBP_O	(0)
#define BITBLTBUF_SBW_O	(16)
#define BITBLTBUF_SPSM_O	(24)
#define BITBLTBUF_DBP_O	(32)
#define BITBLTBUF_DBW_O	(48)
#define BITBLTBUF_DPSM_O	(56)
// BITBLTBUF instances



//CLAMP_x
typedef struct {
	 u_long WMS :2;	// 
	 u_long WMT :2;	// 
	 u_long MINU:10;	// 
	 u_long MAXU:10;	// 
	 u_long MINV:10;	// 
	 u_long MAXV:10;	// 
	 u_long pad0:20;	// padding
} stCLAMP_x;
// CLAMP_x SET macro
#define LIB_SET_CLAMP_1	LIB_SET_CLAMP_x
#define LIB_SET_CLAMP_2	LIB_SET_CLAMP_x
#define LIB_SET_CLAMP_x(WMS, WMT, MINU, MAXU, MINV, MAXV) \
	(((u_long)(WMS) << 0)      | ((u_long)(WMT) << 2)      |\
	 ((u_long)(MINU) << 4)     | ((u_long)(MAXU) << 14)    |\
	 ((u_long)(MINV) << 24)    | ((u_long)(MAXV) << 34))
#define CLAMP_x_WMS_M	(0x3<<0)
#define CLAMP_x_WMT_M	(0x3<<2)
#define CLAMP_x_MINU_M	(0x3FF<<4)
#define CLAMP_x_MAXU_M	(0x3FF<<14)
#define CLAMP_x_MINV_M	(0x3FF<<24)
#define CLAMP_x_MAXV_M	(0x3FF<<34)

#define CLAMP_x_WMS_O	(0)
#define CLAMP_x_WMT_O	(2)
#define CLAMP_x_MINU_O	(4)
#define CLAMP_x_MAXU_O	(14)
#define CLAMP_x_MINV_O	(24)
#define CLAMP_x_MAXV_O	(34)
// CLAMP_x instances



//COLCLAMP
typedef struct {
	 u_long CLAMP:1;	// 
	 u_long pad0 :63;	// padding
} stCOLCLAMP;
// COLCLAMP SET macro
#define LIB_SET_COLCLAMP(CLAMP) \
	(((u_long)(CLAMP) << 0))
#define COLCLAMP_CLAMP_M	(0x1<<0)

#define COLCLAMP_CLAMP_O	(0)
// COLCLAMP instances



//DIMX
typedef struct {
	 u_long DM00 :3;	// dither matrix (0,0)
	 u_long pad0 :1;	// padding
	 u_long DM01 :3;	// dither matrix (0,1)
	 u_long pad1 :1;	// padding
	 u_long DM02 :3;	// dither matrix (0,2)
	 u_long pad2 :1;	// padding
	 u_long DM03 :3;	// dither matrix (0,3)
	 u_long pad3 :1;	// padding
	 u_long DM10 :3;	// dither matrix (1,0)
	 u_long pad4 :1;	// padding
	 u_long DM11 :3;	// dither matrix (1,1)
	 u_long pad5 :1;	// padding
	 u_long DM12 :3;	// dither matrix (1,2)
	 u_long pad6 :1;	// padding
	 u_long DM13 :3;	// dither matrix (1,3)
	 u_long pad7 :1;	// padding
	 u_long DM20 :3;	// dither matrix (2,0)
	 u_long pad8 :1;	// padding
	 u_long DM21 :3;	// dither matrix (2,1)
	 u_long pad9 :1;	// padding
	 u_long DM22 :3;	// dither matrix (2,2)
	 u_long pad10:1;	// padding
	 u_long DM23 :3;	// dither matrix (2,3)
	 u_long pad11:1;	// padding
	 u_long DM30 :3;	// dither matrix (3,0)
	 u_long pad12:1;	// padding
	 u_long DM31 :3;	// dither matrix (3,1)
	 u_long pad13:1;	// padding
	 u_long DM32 :3;	// dither matrix (3,2)
	 u_long pad14:1;	// padding
	 u_long DM33 :3;	// dither matrix (3,3)
	 u_long pad15:1;	// padding
} stDIMX;
// DIMX SET macro
#define LIB_SET_DIMX(DM00, DM01, DM02, DM03, DM10, DM11, DM12, DM13, DM20, DM21, DM22, DM23, DM30, DM31, DM32, DM33) \
	(((u_long)(DM00) << 0)        | ((u_long)(DM01) << 4)        |\
	 ((u_long)(DM02) << 8)        | ((u_long)(DM03) << 12)       |\
	 ((u_long)(DM10) << 16)       | ((u_long)(DM11) << 20)       |\
	 ((u_long)(DM12) << 24)       | ((u_long)(DM13) << 28)       |\
	 ((u_long)(DM20) << 32)       | ((u_long)(DM21) << 36)       |\
	 ((u_long)(DM22) << 40)       | ((u_long)(DM23) << 44)       |\
	 ((u_long)(DM30) << 48)       | ((u_long)(DM31) << 52)       |\
	 ((u_long)(DM32) << 56)       | ((u_long)(DM33) << 60))
#define DIMX_DM00_M	(0x7<<0)
#define DIMX_DM01_M	(0x7<<4)
#define DIMX_DM02_M	(0x7<<8)
#define DIMX_DM03_M	(0x7<<12)
#define DIMX_DM10_M	(0x7<<16)
#define DIMX_DM11_M	(0x7<<20)
#define DIMX_DM12_M	(0x7<<24)
#define DIMX_DM13_M	(0x7<<28)
#define DIMX_DM20_M	(0x7<<32)
#define DIMX_DM21_M	(0x7<<36)
#define DIMX_DM22_M	(0x7<<40)
#define DIMX_DM23_M	(0x7<<44)
#define DIMX_DM30_M	(0x7<<48)
#define DIMX_DM31_M	(0x7<<52)
#define DIMX_DM32_M	(0x7<<56)
#define DIMX_DM33_M	(0x7<<60)

#define DIMX_DM00_O	(0)
#define DIMX_DM01_O	(4)
#define DIMX_DM02_O	(8)
#define DIMX_DM03_O	(12)
#define DIMX_DM10_O	(16)
#define DIMX_DM11_O	(20)
#define DIMX_DM12_O	(24)
#define DIMX_DM13_O	(28)
#define DIMX_DM20_O	(32)
#define DIMX_DM21_O	(36)
#define DIMX_DM22_O	(40)
#define DIMX_DM23_O	(44)
#define DIMX_DM30_O	(48)
#define DIMX_DM31_O	(52)
#define DIMX_DM32_O	(56)
#define DIMX_DM33_O	(60)
// DIMX instances



//DTHE
typedef struct {
	 u_long DTHE:1;	// 
	 u_long pad0:63;	// padding
} stDTHE;
// DTHE SET macro
#define LIB_SET_DTHE(DTHE) \
	(((u_long)(DTHE) << 0))
#define DTHE_DTHE_M	(0x1<<0)

#define DTHE_DTHE_O	(0)
// DTHE instances



//FBA_x
typedef struct {
	 u_long FBA :1;	// 
	 u_long pad0:63;	// padding
} stFBA_x;
// FBA_x SET macro
#define LIB_SET_FBA_1	LIB_SET_FBA_x
#define LIB_SET_FBA_2	LIB_SET_FBA_x
#define LIB_SET_FBA_x(FBA) \
	(((u_long)(FBA) << 0))
#define FBA_x_FBA_M	(0x1<<0)

#define FBA_x_FBA_O	(0)
// FBA_x instances



//FINISH
typedef struct {
	 u_long FINISH:64;	// 
} stFINISH;
// FINISH SET macro
#define LIB_SET_FINISH(FINISH) \
	(((u_long)(FINISH) << 0))
#define FINISH_FINISH_M	(0xFFFFFFFFFFFFFFFF<<0)

#define FINISH_FINISH_O	(0)
// FINISH instances



//FOG
typedef struct {
	 u_long pad0:56;	// padding
	 u_long F   :8;	// fog value
} stFOG;
// FOG SET macro
#define LIB_SET_FOG(F) \
	(((u_long)(F) << 56))
#define FOG_F_M	(0xFF<<56)

#define FOG_F_O	(56)
// FOG instances



//FOGCOL
typedef struct {
	 u_long FCR :8;	// fog color (r)
	 u_long FCG :8;	// fog color (g)
	 u_long FCB :8;	// fog color (b)
	 u_long pad0:40;	// padding
} stFOGCOL;
// FOGCOL SET macro
#define LIB_SET_FOGCOL(FCR, FCG, FCB) \
	(((u_long)(FCR) << 0)    | ((u_long)(FCG) << 8)    |\
	 ((u_long)(FCB) << 16))
#define FOGCOL_FCR_M	(0xFF<<0)
#define FOGCOL_FCG_M	(0xFF<<8)
#define FOGCOL_FCB_M	(0xFF<<16)

#define FOGCOL_FCR_O	(0)
#define FOGCOL_FCG_O	(8)
#define FOGCOL_FCB_O	(16)
// FOGCOL instances



//FRAME_x
typedef struct {
	 u_long FBP  :9;	// 
	 u_long pad0 :7;	// padding
	 u_long FBW  :6;	// 
	 u_long pad1 :2;	// padding
	 u_long PSM  :6;	// 
	 u_long pad2 :2;	// padding
	 u_long FBMSK:32;	// 
} stFRAME_x;
// FRAME_x SET macro
#define LIB_SET_FRAME_1	LIB_SET_FRAME_x
#define LIB_SET_FRAME_2	LIB_SET_FRAME_x
#define LIB_SET_FRAME_x(FBP, FBW, PSM, FBMSK) \
	(((u_long)(FBP) << 0)      | ((u_long)(FBW) << 16)     |\
	 ((u_long)(PSM) << 24)     | ((u_long)(FBMSK) << 32))
#define FRAME_x_FBP_M	(0x1FF<<0)
#define FRAME_x_FBW_M	(0x3F<<16)
#define FRAME_x_PSM_M	(0x3F<<24)
#define FRAME_x_FBMSK_M	(0xFFFFFFFF<<32)

#define FRAME_x_FBP_O	(0)
#define FRAME_x_FBW_O	(16)
#define FRAME_x_PSM_O	(24)
#define FRAME_x_FBMSK_O	(32)
// FRAME_x instances



//HWREG
typedef struct {
	 u_long DATA:64;	// host->local transmission data
} stHWREG;
// HWREG SET macro
#define LIB_SET_HWREG(DATA) \
	(((u_long)(DATA) << 0))
#define HWREG_DATA_M	(0xFFFFFFFFFFFFFFFF<<0)

#define HWREG_DATA_O	(0)
// HWREG instances



//LABEL
typedef struct {
	 u_long ID   :32;	// value to be written to siglblid register
	 u_long IDMSK:32;	// whether or not corresponding siglblid bit is updated
} stLABEL;
// LABEL SET macro
#define LIB_SET_LABEL(ID, IDMSK) \
	(((u_long)(ID) << 0)       | ((u_long)(IDMSK) << 32))
#define LABEL_ID_M	(0xFFFFFFFF<<0)
#define LABEL_IDMSK_M	(0xFFFFFFFF<<32)

#define LABEL_ID_O	(0)
#define LABEL_IDMSK_O	(32)
// LABEL instances



//MIPTBPx_y
typedef struct {
	 u_long TBP1:14;	// mipmap level 1/4 texture base pointer
	 u_long TBW1:6;	// mipmap level 1/4 texture base width
	 u_long TBP2:14;	// mipmap level 2/5 texture base pointer
	 u_long TBW2:6;	// mipmap level 2/5 texture base width
	 u_long TBP3:14;	// mipmap level 3/6 texture base pointer
	 u_long TBW3:6;	// mipmap level 3/6 texture base width
	 u_long pad0:4;	// padding
} stMIPTBPx_y;
// MIPTBPx_y SET macro
#define LIB_SET_MIPTBP1_1	LIB_SET_MIPTBPx_y
#define LIB_SET_MIPTBP1_2	LIB_SET_MIPTBPx_y
#define LIB_SET_MIPTBP2_1	LIB_SET_MIPTBPx_y
#define LIB_SET_MIPTBP2_2	LIB_SET_MIPTBPx_y
#define LIB_SET_MIPTBPx_y(TBP1, TBW1, TBP2, TBW2, TBP3, TBW3) \
	(((u_long)(TBP1) << 0)       | ((u_long)(TBW1) << 14)      |\
	 ((u_long)(TBP2) << 20)      | ((u_long)(TBW2) << 34)      |\
	 ((u_long)(TBP3) << 40)      | ((u_long)(TBW3) << 54))
#define MIPTBPx_y_TBP1_M	(0x3FFF<<0)
#define MIPTBPx_y_TBW1_M	(0x3F<<14)
#define MIPTBPx_y_TBP2_M	(0x3FFF<<20)
#define MIPTBPx_y_TBW2_M	(0x3F<<34)
#define MIPTBPx_y_TBP3_M	(0x3FFF<<40)
#define MIPTBPx_y_TBW3_M	(0x3F<<54)

#define MIPTBPx_y_TBP1_O	(0)
#define MIPTBPx_y_TBW1_O	(14)
#define MIPTBPx_y_TBP2_O	(20)
#define MIPTBPx_y_TBW2_O	(34)
#define MIPTBPx_y_TBP3_O	(40)
#define MIPTBPx_y_TBW3_O	(54)
// MIPTBPx_y instances



//PABE
typedef struct {
	 u_long PABE:1;	// alpha blending control in units of pixels
	 u_long pad0:63;	// padding
} stPABE;
// PABE SET macro
#define LIB_SET_PABE(PABE) \
	(((u_long)(PABE) << 0))
#define PABE_PABE_M	(0x1<<0)

#define PABE_PABE_O	(0)
// PABE instances



//PRIM
typedef struct {
	 u_long PRIM:3;	// type of drawing primitive
	 u_long IIP :1;	// shading method
	 u_long TME :1;	// texture mapping
	 u_long FGE :1;	// fogging
	 u_long ABE :1;	// alpha blending
	 u_long AA1 :1;	// pass antialiasing
	 u_long FST :1;	// method of specifying texture coordinates
	 u_long CTXT:1;	// context
	 u_long FIX :1;	// fragment value control
	 u_long pad0:53;	// padding
} stPRIM;
// PRIM SET macro
#define LIB_SET_PRIM(PRIM, IIP, TME, FGE, ABE, AA1, FST, CTXT, FIX) \
	(((u_long)(PRIM) << 0)  | ((u_long)(IIP) << 3)   |\
	 ((u_long)(TME) << 4)   | ((u_long)(FGE) << 5)   |\
	 ((u_long)(ABE) << 6)   | ((u_long)(AA1) << 7)   |\
	 ((u_long)(FST) << 8)   | ((u_long)(CTXT) << 9)  |\
	 ((u_long)(FIX) << 10))
#define PRIM_PRIM_M	(0x7<<0)
#define PRIM_IIP_M	(0x1<<3)
#define PRIM_TME_M	(0x1<<4)
#define PRIM_FGE_M	(0x1<<5)
#define PRIM_ABE_M	(0x1<<6)
#define PRIM_AA1_M	(0x1<<7)
#define PRIM_FST_M	(0x1<<8)
#define PRIM_CTXT_M	(0x1<<9)
#define PRIM_FIX_M	(0x1<<10)

#define PRIM_PRIM_O	(0)
#define PRIM_IIP_O	(3)
#define PRIM_TME_O	(4)
#define PRIM_FGE_O	(5)
#define PRIM_ABE_O	(6)
#define PRIM_AA1_O	(7)
#define PRIM_FST_O	(8)
#define PRIM_CTXT_O	(9)
#define PRIM_FIX_O	(10)
// PRIM instances



//PRMODE
typedef struct {
	 u_long pad0:3;	// padding
	 u_long IIP :1;	// shading method
	 u_long TME :1;	// texture mapping
	 u_long FGE :1;	// fogging
	 u_long ABE :1;	// alpha blending
	 u_long AA1 :1;	// pass antialiasing
	 u_long FST :1;	// method of specifying texture coordinates
	 u_long CTXT:1;	// context
	 u_long FIX :1;	// fragment value control
	 u_long pad1:53;	// padding
} stPRMODE;
// PRMODE SET macro
#define LIB_SET_PRMODE(IIP, TME, FGE, ABE, AA1, FST, CTXT, FIX) \
	(((u_long)(IIP) << 3)   | ((u_long)(TME) << 4)   |\
	 ((u_long)(FGE) << 5)   | ((u_long)(ABE) << 6)   |\
	 ((u_long)(AA1) << 7)   | ((u_long)(FST) << 8)   |\
	 ((u_long)(CTXT) << 9)  | ((u_long)(FIX) << 10))
#define PRMODE_IIP_M	(0x1<<3)
#define PRMODE_TME_M	(0x1<<4)
#define PRMODE_FGE_M	(0x1<<5)
#define PRMODE_ABE_M	(0x1<<6)
#define PRMODE_AA1_M	(0x1<<7)
#define PRMODE_FST_M	(0x1<<8)
#define PRMODE_CTXT_M	(0x1<<9)
#define PRMODE_FIX_M	(0x1<<10)

#define PRMODE_IIP_O	(3)
#define PRMODE_TME_O	(4)
#define PRMODE_FGE_O	(5)
#define PRMODE_ABE_O	(6)
#define PRMODE_AA1_O	(7)
#define PRMODE_FST_O	(8)
#define PRMODE_CTXT_O	(9)
#define PRMODE_FIX_O	(10)
// PRMODE instances



//PRMODECONT
typedef struct {
	 u_long AC  :1;	// 
	 u_long pad0:63;	// padding
} stPRMODECONT;
// PRMODECONT SET macro
#define LIB_SET_PRMODECONT(AC) \
	(((u_long)(AC) << 0))
#define PRMODECONT_AC_M	(0x1<<0)

#define PRMODECONT_AC_O	(0)
// PRMODECONT instances



//RGBAQ
typedef struct {
	 u_long R:8;	// luminance value of r element of vertex color
	 u_long G:8;	// luminance value of g element of vertex color
	 u_long B:8;	// luminance value of b element of vertex color
	 u_long A:8;	// alpha value of vertex (0x80=1.0)
	 u_long Q:32;	// normalized texture coordinates
} stRGBAQ;
// RGBAQ SET macro
#define LIB_SET_RGBAQ(R, G, B, A, Q) \
	(((u_long)(R) << 0)    | ((u_long)(G) << 8)    |\
	 ((u_long)(B) << 16)   | ((u_long)(A) << 24)   |\
	 ((u_long)(Q) << 32))
#define RGBAQ_R_M	(0xFF<<0)
#define RGBAQ_G_M	(0xFF<<8)
#define RGBAQ_B_M	(0xFF<<16)
#define RGBAQ_A_M	(0xFF<<24)
#define RGBAQ_Q_M	(0xFFFFFFFF<<32)

#define RGBAQ_R_O	(0)
#define RGBAQ_G_O	(8)
#define RGBAQ_B_O	(16)
#define RGBAQ_A_O	(24)
#define RGBAQ_Q_O	(32)
// RGBAQ instances



//SCANMSK
typedef struct {
	 u_long MSK :2;	// raster address mask setting
	 u_long pad0:62;	// padding
} stSCANMSK;
// SCANMSK SET macro
#define LIB_SET_SCANMSK(MSK) \
	(((u_long)(MSK) << 0))
#define SCANMSK_MSK_M	(0x3<<0)

#define SCANMSK_MSK_O	(0)
// SCANMSK instances



//SCISSOR_x
typedef struct {
	 u_long SCAX0:11;	// 
	 u_long pad0 :5;	// padding
	 u_long SCAX1:11;	// 
	 u_long pad1 :5;	// padding
	 u_long SCAY0:11;	// 
	 u_long pad2 :5;	// padding
	 u_long SCAY1:11;	// 
	 u_long pad3 :5;	// padding
} stSCISSOR_x;
// SCISSOR_x SET macro
#define LIB_SET_SCISSOR_1	LIB_SET_SCISSOR_x
#define LIB_SET_SCISSOR_2	LIB_SET_SCISSOR_x
#define LIB_SET_SCISSOR_x(SCAX0, SCAX1, SCAY0, SCAY1) \
	(((u_long)(SCAX0) << 0)     | ((u_long)(SCAX1) << 16)    |\
	 ((u_long)(SCAY0) << 32)    | ((u_long)(SCAY1) << 48))
#define SCISSOR_x_SCAX0_M	(0x7FF<<0)
#define SCISSOR_x_SCAX1_M	(0x7FF<<16)
#define SCISSOR_x_SCAY0_M	(0x7FF<<32)
#define SCISSOR_x_SCAY1_M	(0x7FF<<48)

#define SCISSOR_x_SCAX0_O	(0)
#define SCISSOR_x_SCAX1_O	(16)
#define SCISSOR_x_SCAY0_O	(32)
#define SCISSOR_x_SCAY1_O	(48)
// SCISSOR_x instances



//SIGNAL
typedef struct {
	 u_long ID   :32;	// value to be written to siglblid register
	 u_long IDMSK:32;	// whether or not siglblid bit is updated
} stSIGNAL;
// SIGNAL SET macro
#define LIB_SET_SIGNAL(ID, IDMSK) \
	(((u_long)(ID) << 0)       | ((u_long)(IDMSK) << 32))
#define SIGNAL_ID_M	(0xFFFFFFFF<<0)
#define SIGNAL_IDMSK_M	(0xFFFFFFFF<<32)

#define SIGNAL_ID_O	(0)
#define SIGNAL_IDMSK_O	(32)
// SIGNAL instances



//ST
typedef struct {
	 u_long S:32;	// texture coordinate s value
	 u_long T:32;	// texture coordinate t value
} stST;
// ST SET macro
#define LIB_SET_ST(S, T) \
	(((u_long)(S) << 0)    | ((u_long)(T) << 32))
#define ST_S_M	(0xFFFFFFFF<<0)
#define ST_T_M	(0xFFFFFFFF<<32)

#define ST_S_O	(0)
#define ST_T_O	(32)
// ST instances



//TEST_x
typedef struct {
	 u_long ATE  :1;	// 
	 u_long ATST :3;	// 
	 u_long AREF :8;	// 
	 u_long AFAIL:2;	// 
	 u_long DATE :1;	// 
	 u_long DATM :1;	// 
	 u_long ZTE  :1;	// 
	 u_long ZTST :2;	// 
	 u_long pad0 :45;	// padding
} stTEST_x;
// TEST_x SET macro
#define LIB_SET_TEST_1	LIB_SET_TEST_x
#define LIB_SET_TEST_2	LIB_SET_TEST_x
#define LIB_SET_TEST_x(ATE, ATST, AREF, AFAIL, DATE, DATM, ZTE, ZTST) \
	(((u_long)(ATE) << 0)    | ((u_long)(ATST) << 1)   |\
	 ((u_long)(AREF) << 4)   | ((u_long)(AFAIL) << 12) |\
	 ((u_long)(DATE) << 14)  | ((u_long)(DATM) << 15)  |\
	 ((u_long)(ZTE) << 16)   | ((u_long)(ZTST) << 17))
#define TEST_x_ATE_M	(0x1<<0)
#define TEST_x_ATST_M	(0x7<<1)
#define TEST_x_AREF_M	(0xFF<<4)
#define TEST_x_AFAIL_M	(0x3<<12)
#define TEST_x_DATE_M	(0x1<<14)
#define TEST_x_DATM_M	(0x1<<15)
#define TEST_x_ZTE_M	(0x1<<16)
#define TEST_x_ZTST_M	(0x3<<17)

#define TEST_x_ATE_O	(0)
#define TEST_x_ATST_O	(1)
#define TEST_x_AREF_O	(4)
#define TEST_x_AFAIL_O	(12)
#define TEST_x_DATE_O	(14)
#define TEST_x_DATM_O	(15)
#define TEST_x_ZTE_O	(16)
#define TEST_x_ZTST_O	(17)
// TEST_x instances



//TEX0_x
typedef struct {
	 u_long TBP0:14;	// texture base pointer
	 u_long TBW :6;	// texture buffer width
	 u_long PSM :6;	// texture pixel storage format
	 u_long TW  :4;	// texture width
	 u_long TH  :4;	// texture height
	 u_long TCC :1;	// texture color component
	 u_long TFX :2;	// texture function
	 u_long CBP :14;	// clut buffer base pointer
	 u_long CPSM:4;	// clut pixel storage format
	 u_long CSM :1;	// clut storage mode
	 u_long CSA :5;	// clut entry offset
	 u_long CLD :3;	// clut buffer load control
} stTEX0_x;
// TEX0_x SET macro
#define LIB_SET_TEX0_1	LIB_SET_TEX0_x
#define LIB_SET_TEX0_2	LIB_SET_TEX0_x
#define LIB_SET_TEX0_x(TBP0, TBW, PSM, TW, TH, TCC, TFX, CBP, CPSM, CSM, CSA, CLD) \
	(((u_long)(TBP0) << 0)      | ((u_long)(TBW) << 14)      |\
	 ((u_long)(PSM) << 20)      | ((u_long)(TW) << 26)       |\
	 ((u_long)(TH) << 30)       | ((u_long)(TCC) << 34)      |\
	 ((u_long)(TFX) << 35)      | ((u_long)(CBP) << 37)      |\
	 ((u_long)(CPSM) << 51)     | ((u_long)(CSM) << 55)      |\
	 ((u_long)(CSA) << 56)      | ((u_long)(CLD) << 61))
#define TEX0_x_TBP0_M	(0x3FFF<<0)
#define TEX0_x_TBW_M	(0x3F<<14)
#define TEX0_x_PSM_M	(0x3F<<20)
#define TEX0_x_TW_M	(0xF<<26)
#define TEX0_x_TH_M	(0xF<<30)
#define TEX0_x_TCC_M	(0x1<<34)
#define TEX0_x_TFX_M	(0x3<<35)
#define TEX0_x_CBP_M	(0x3FFF<<37)
#define TEX0_x_CPSM_M	(0xF<<51)
#define TEX0_x_CSM_M	(0x1<<55)
#define TEX0_x_CSA_M	(0x1F<<56)
#define TEX0_x_CLD_M	(0x7<<61)

#define TEX0_x_TBP0_O	(0)
#define TEX0_x_TBW_O	(14)
#define TEX0_x_PSM_O	(20)
#define TEX0_x_TW_O	(26)
#define TEX0_x_TH_O	(30)
#define TEX0_x_TCC_O	(34)
#define TEX0_x_TFX_O	(35)
#define TEX0_x_CBP_O	(37)
#define TEX0_x_CPSM_O	(51)
#define TEX0_x_CSM_O	(55)
#define TEX0_x_CSA_O	(56)
#define TEX0_x_CLD_O	(61)
// TEX0_x instances



//TEX1_x
typedef struct {
	 u_long LCM :1;	// lod calculation method
	 u_long pad0:1;	// padding
	 u_long MXL :3;	// maximum mip level
	 u_long MMAG:1;	// filter when texture is expanded
	 u_long MMIN:3;	// filter when texture is reduced
	 u_long MTBA:1;	// base address specification of mipmap texture level 1 or more
	 u_long pad1:9;	// padding
	 u_long L   :2;	// lod parameter l value
	 u_long pad2:11;	// padding
	 u_long K   :12;	// lod parameter k value
	 u_long pad3:20;	// padding
} stTEX1_x;
// TEX1_x SET macro
#define LIB_SET_TEX1_1	LIB_SET_TEX1_x
#define LIB_SET_TEX1_2	LIB_SET_TEX1_x
#define LIB_SET_TEX1_x(LCM, MXL, MMAG, MMIN, MTBA, L, K) \
	(((u_long)(LCM) << 0)      | ((u_long)(MXL) << 2)      |\
	 ((u_long)(MMAG) << 5)     | ((u_long)(MMIN) << 6)     |\
	 ((u_long)(MTBA) << 9)     | ((u_long)(L) << 19)       |\
	 ((u_long)(K) << 32))
#define TEX1_x_LCM_M	(0x1<<0)
#define TEX1_x_MXL_M	(0x7<<2)
#define TEX1_x_MMAG_M	(0x1<<5)
#define TEX1_x_MMIN_M	(0x7<<6)
#define TEX1_x_MTBA_M	(0x1<<9)
#define TEX1_x_L_M	(0x3<<19)
#define TEX1_x_K_M	(0xFFF<<32)

#define TEX1_x_LCM_O	(0)
#define TEX1_x_MXL_O	(2)
#define TEX1_x_MMAG_O	(5)
#define TEX1_x_MMIN_O	(6)
#define TEX1_x_MTBA_O	(9)
#define TEX1_x_L_O	(19)
#define TEX1_x_K_O	(32)
// TEX1_x instances



//TEX2_x
typedef struct {
	 u_long pad0:20;	// padding
	 u_long PSM :6;	// texture pixel storage format
	 u_long pad1:11;	// padding
	 u_long CBP :14;	// clut buffer base pointer
	 u_long CPSM:4;	// clut entry storage format
	 u_long CSM :1;	// clut storage mode
	 u_long CSA :5;	// clut entry offset
	 u_long CLD :3;	// clut buffer load control
} stTEX2_x;
// TEX2_x SET macro
#define LIB_SET_TEX2_1	LIB_SET_TEX2_x
#define LIB_SET_TEX2_2	LIB_SET_TEX2_x
#define LIB_SET_TEX2_x(PSM, CBP, CPSM, CSM, CSA, CLD) \
	(((u_long)(PSM) << 20)      | ((u_long)(CBP) << 37)      |\
	 ((u_long)(CPSM) << 51)     | ((u_long)(CSM) << 55)      |\
	 ((u_long)(CSA) << 56)      | ((u_long)(CLD) << 61))
#define TEX2_x_PSM_M	(0x3F<<20)
#define TEX2_x_CBP_M	(0x3FFF<<37)
#define TEX2_x_CPSM_M	(0xF<<51)
#define TEX2_x_CSM_M	(0x1<<55)
#define TEX2_x_CSA_M	(0x1F<<56)
#define TEX2_x_CLD_M	(0x7<<61)

#define TEX2_x_PSM_O	(20)
#define TEX2_x_CBP_O	(37)
#define TEX2_x_CPSM_O	(51)
#define TEX2_x_CSM_O	(55)
#define TEX2_x_CSA_O	(56)
#define TEX2_x_CLD_O	(61)
// TEX2_x instances



//TEXA
typedef struct {
	 u_long TA0 :8;	// 
	 u_long pad0:7;	// padding
	 u_long AEM :1;	// method of expanding texture alpha
	 u_long pad1:16;	// padding
	 u_long TA1 :8;	// 
	 u_long pad2:24;	// padding
} stTEXA;
// TEXA SET macro
#define LIB_SET_TEXA(TA0, AEM, TA1) \
	(((u_long)(TA0) << 0)      | ((u_long)(AEM) << 15)     |\
	 ((u_long)(TA1) << 32))
#define TEXA_TA0_M	(0xFF<<0)
#define TEXA_AEM_M	(0x1<<15)
#define TEXA_TA1_M	(0xFF<<32)

#define TEXA_TA0_O	(0)
#define TEXA_AEM_O	(15)
#define TEXA_TA1_O	(32)
// TEXA instances



//TEXCLUT
typedef struct {
	 u_long CBW :6;	// clut buffer width
	 u_long COU :6;	// clut offset u
	 u_long COV :10;	// clut offset v
	 u_long pad0:42;	// padding
} stTEXCLUT;
// TEXCLUT SET macro
#define LIB_SET_TEXCLUT(CBW, COU, COV) \
	(((u_long)(CBW) << 0)    | ((u_long)(COU) << 6)    |\
	 ((u_long)(COV) << 12))
#define TEXCLUT_CBW_M	(0x3F<<0)
#define TEXCLUT_COU_M	(0x3F<<6)
#define TEXCLUT_COV_M	(0x3FF<<12)

#define TEXCLUT_CBW_O	(0)
#define TEXCLUT_COU_O	(6)
#define TEXCLUT_COV_O	(12)
// TEXCLUT instances



//TEXFLUSH
typedef struct {
	 u_long TEXTFLUSH:64;	// 
} stTEXFLUSH;
// TEXFLUSH SET macro
#define LIB_SET_TEXFLUSH(TEXTFLUSH) \
	(((u_long)(TEXTFLUSH) << 0))
#define TEXFLUSH_TEXTFLUSH_M	(0xFFFFFFFFFFFFFFFF<<0)

#define TEXFLUSH_TEXTFLUSH_O	(0)
// TEXFLUSH instances



//TRXDIR
typedef struct {
	 u_long XDIR:2;	// transmission direction is specified and transmission is started
	 u_long pad0:62;	// padding
} stTRXDIR;
// TRXDIR SET macro
#define LIB_SET_TRXDIR(XDIR) \
	(((u_long)(XDIR) << 0))
#define TRXDIR_XDIR_M	(0x3<<0)

#define TRXDIR_XDIR_O	(0)
// TRXDIR instances



//TRXPOS
typedef struct {
	 u_long SSAX:11;	// x coordinate of upper left point of source rectangular area
	 u_long pad0:5;	// padding
	 u_long SSAY:11;	// y coordinate of upper left point of source rectangular area
	 u_long pad1:5;	// padding
	 u_long DSAX:11;	// x coordinate of upper left point of destination rectangular area
	 u_long pad2:5;	// padding
	 u_long DSAY:11;	// y coordinate of upper left point of destination rectangular area
	 u_long DIR :2;	// pixel transmission order
	 u_long pad3:3;	// padding
} stTRXPOS;
// TRXPOS SET macro
#define LIB_SET_TRXPOS(SSAX, SSAY, DSAX, DSAY, DIR) \
	(((u_long)(SSAX) << 0)       | ((u_long)(SSAY) << 16)      |\
	 ((u_long)(DSAX) << 32)      | ((u_long)(DSAY) << 48)      |\
	 ((u_long)(DIR) << 59))
#define TRXPOS_SSAX_M	(0x7FF<<0)
#define TRXPOS_SSAY_M	(0x7FF<<16)
#define TRXPOS_DSAX_M	(0x7FF<<32)
#define TRXPOS_DSAY_M	(0x7FF<<48)
#define TRXPOS_DIR_M	(0x3<<59)

#define TRXPOS_SSAX_O	(0)
#define TRXPOS_SSAY_O	(16)
#define TRXPOS_DSAX_O	(32)
#define TRXPOS_DSAY_O	(48)
#define TRXPOS_DIR_O	(59)
// TRXPOS instances



//TRXREG
typedef struct {
	 u_long RRW :12;	// width of transmission area
	 u_long pad0:20;	// padding
	 u_long RRH :12;	// height of transmission area
	 u_long pad1:20;	// padding
} stTRXREG;
// TRXREG SET macro
#define LIB_SET_TRXREG(RRW, RRH) \
	(((u_long)(RRW) << 0)      | ((u_long)(RRH) << 32))
#define TRXREG_RRW_M	(0xFFF<<0)
#define TRXREG_RRH_M	(0xFFF<<32)

#define TRXREG_RRW_O	(0)
#define TRXREG_RRH_O	(32)
// TRXREG instances



//UV
typedef struct {
	 u_long U   :14;	// texel u coordinate
	 u_long pad0:2;	// padding
	 u_long V   :14;	// texel v coordinate
	 u_long pad1:34;	// padding
} stUV;
// UV SET macro
#define LIB_SET_UV(U, V) \
	(((u_long)(U) << 0)       | ((u_long)(V) << 16))
#define UV_U_M	(0x3FFF<<0)
#define UV_V_M	(0x3FFF<<16)

#define UV_U_O	(0)
#define UV_V_O	(16)
// UV instances



//XYOFFSET_x
typedef struct {
	 u_long OFX :16;	// 
	 u_long pad0:16;	// padding
	 u_long OFY :16;	// 
	 u_long pad1:16;	// padding
} stXYOFFSET_x;
// XYOFFSET_x SET macro
#define LIB_SET_XYOFFSET_1	LIB_SET_XYOFFSET_x
#define LIB_SET_XYOFFSET_2	LIB_SET_XYOFFSET_x
#define LIB_SET_XYOFFSET_x(OFX, OFY) \
	(((u_long)(OFX) << 0)      | ((u_long)(OFY) << 32))
#define XYOFFSET_x_OFX_M	(0xFFFF<<0)
#define XYOFFSET_x_OFY_M	(0xFFFF<<32)

#define XYOFFSET_x_OFX_O	(0)
#define XYOFFSET_x_OFY_O	(32)
// XYOFFSET_x instances



//XYZx
typedef struct {
	 u_long X:16;	// vertex cordinate value (x)
	 u_long Y:16;	// vertex cordinate value (y)
	 u_long Z:32;	// vertex cordinate value (z)
} stXYZx;
// XYZx SET macro
#define LIB_SET_XYZ2	LIB_SET_XYZx
#define LIB_SET_XYZ3	LIB_SET_XYZx
#define LIB_SET_XYZx(X, Y, Z) \
	(((u_long)(X) << 0)    | ((u_long)(Y) << 16)   |\
	 ((u_long)(Z) << 32))
#define XYZx_X_M	(0xFFFF<<0)
#define XYZx_Y_M	(0xFFFF<<16)
#define XYZx_Z_M	(0xFFFFFFFF<<32)

#define XYZx_X_O	(0)
#define XYZx_Y_O	(16)
#define XYZx_Z_O	(32)
// XYZx instances



//XYZFx
typedef struct {
	 u_long X:16;	// vertex cordinate value (x)
	 u_long Y:16;	// vertex cordinate value (y)
	 u_long Z:24;	// vertex cordinate value (z)
	 u_long F:8;	// fog coefficient
} stXYZFx;
// XYZFx SET macro
#define LIB_SET_XYZF2	LIB_SET_XYZFx
#define LIB_SET_XYZF3	LIB_SET_XYZFx
#define LIB_SET_XYZFx(X, Y, Z, F) \
	(((u_long)(X) << 0)      | ((u_long)(Y) << 16)     |\
	 ((u_long)(Z) << 32)     | ((u_long)(F) << 56))
#define XYZFx_X_M	(0xFFFF<<0)
#define XYZFx_Y_M	(0xFFFF<<16)
#define XYZFx_Z_M	(0xFFFFFF<<32)
#define XYZFx_F_M	(0xFF<<56)

#define XYZFx_X_O	(0)
#define XYZFx_Y_O	(16)
#define XYZFx_Z_O	(32)
#define XYZFx_F_O	(56)
// XYZFx instances



//ZBUF_x
typedef struct {
	 u_long ZBP :9;	// 
	 u_long pad0:15;	// padding
	 u_long PSM :4;	// 
	 u_long pad1:4;	// padding
	 u_long ZMSK:1;	// 
	 u_long pad2:31;	// padding
} stZBUF_x;
// ZBUF_x SET macro
#define LIB_SET_ZBUF_1	LIB_SET_ZBUF_x
#define LIB_SET_ZBUF_2	LIB_SET_ZBUF_x
#define LIB_SET_ZBUF_x(ZBP, PSM, ZMSK) \
	(((u_long)(ZBP) << 0)     | ((u_long)(PSM) << 24)    |\
	 ((u_long)(ZMSK) << 32))
#define ZBUF_x_ZBP_M	(0x1FF<<0)
#define ZBUF_x_PSM_M	(0xF<<24)
#define ZBUF_x_ZMSK_M	(0x1<<32)

#define ZBUF_x_ZBP_O	(0)
#define ZBUF_x_PSM_O	(24)
#define ZBUF_x_ZMSK_O	(32)
// ZBUF_x instances



//BGCOLOR
typedef struct {
	 u_long R   :8;	// luminance of red component of background color
	 u_long G   :8;	// luminance of green component of background color
	 u_long B   :8;	// luminance of blue component of background color
	 u_long pad0:40;	// padding
} stBGCOLOR;
// BGCOLOR SET macro
#define LIB_SET_BGCOLOR(R, G, B) \
	(((u_long)(R) << 0)      | ((u_long)(G) << 8)      |\
	 ((u_long)(B) << 16))
#define BGCOLOR_R_M	(0xFF<<0)
#define BGCOLOR_G_M	(0xFF<<8)
#define BGCOLOR_B_M	(0xFF<<16)

#define BGCOLOR_R_O	(0)
#define BGCOLOR_G_O	(8)
#define BGCOLOR_B_O	(16)
// BGCOLOR instances
#define BGCOLOR 	((volatile u_long *)(0x120000e0))



//BUSDIR
typedef struct {
	 u_long DIR :1;	// transmission direction of interface
	 u_long pad0:63;	// padding
} stBUSDIR;
// BUSDIR SET macro
#define LIB_SET_BUSDIR(DIR) \
	(((u_long)(DIR) << 0))
#define BUSDIR_DIR_M	(0x1<<0)

#define BUSDIR_DIR_O	(0)
// BUSDIR instances
#define BUSDIR 	((volatile u_long *)(0x12001040))



//CSR
typedef struct {
	 u_long SIGNAL:1;	// signal event control
	 u_long FINISH:1;	// finish event control
	 u_long HSINT :1;	// hsync interrupt control
	 u_long VSINT :1;	// vsync interrupt control
	 u_long EDWINT:1;	// rect area write (raw) termination interrupt control
	 u_long pad0  :3;	// padding
	 u_long FLUSH :1;	// drawing suspend and fifo clear (enabled during data write)
	 u_long RESET :1;	// gs system reset (enabled during data write)
	 u_long pad1  :2;	// padding
	 u_long NFIELD:1;	// output value of nfield output (huh?)
	 u_long FIELD :1;	// field displayed currently
	 u_long FIFO  :2;	// host interface fifo status
	 u_long REV   :8;	// 
	 u_long ID    :8;	// 
	 u_long pad2  :32;	// padding
} stCSR;
// CSR SET macro
#define LIB_SET_CSR(SIGNAL, FINISH, HSINT, VSINT, EDWINT, FLUSH, RESET, NFIELD, FIELD, FIFO, REV, ID) \
	(((u_long)(SIGNAL) << 0)  | ((u_long)(FINISH) << 1)  |\
	 ((u_long)(HSINT) << 2)   | ((u_long)(VSINT) << 3)   |\
	 ((u_long)(EDWINT) << 4)  | ((u_long)(FLUSH) << 8)   |\
	 ((u_long)(RESET) << 9)   | ((u_long)(NFIELD) << 12) |\
	 ((u_long)(FIELD) << 13)  | ((u_long)(FIFO) << 14)   |\
	 ((u_long)(REV) << 16)    | ((u_long)(ID) << 24))
#define CSR_SIGNAL_M	(0x1<<0)
#define CSR_FINISH_M	(0x1<<1)
#define CSR_HSINT_M	(0x1<<2)
#define CSR_VSINT_M	(0x1<<3)
#define CSR_EDWINT_M	(0x1<<4)
#define CSR_FLUSH_M	(0x1<<8)
#define CSR_RESET_M	(0x1<<9)
#define CSR_NFIELD_M	(0x1<<12)
#define CSR_FIELD_M	(0x1<<13)
#define CSR_FIFO_M	(0x3<<14)
#define CSR_REV_M	(0xFF<<16)
#define CSR_ID_M	(0xFF<<24)

#define CSR_SIGNAL_O	(0)
#define CSR_FINISH_O	(1)
#define CSR_HSINT_O	(2)
#define CSR_VSINT_O	(3)
#define CSR_EDWINT_O	(4)
#define CSR_FLUSH_O	(8)
#define CSR_RESET_O	(9)
#define CSR_NFIELD_O	(12)
#define CSR_FIELD_O	(13)
#define CSR_FIFO_O	(14)
#define CSR_REV_O	(16)
#define CSR_ID_O	(24)
// CSR instances
#define CSR 	((volatile u_long *)(0x12001000))



//DISPFBn
typedef struct {
	 u_long FBP :9;	// base pointer (addr/2048)
	 u_long FBW :6;	// buffer width (width/64)
	 u_long PSM :5;	// pixel storage format
	 u_long pad0:12;	// padding
	 u_long DBX :11;	// top left x position in buffer (pixels)
	 u_long DBY :11;	// top left y position in buffer (pixels)
	 u_long pad1:10;	// padding
} stDISPFBn;
// DISPFBn SET macro
#define LIB_SET_DISPFB1	LIB_SET_DISPFBn
#define LIB_SET_DISPFB2	LIB_SET_DISPFBn
#define LIB_SET_DISPFBn(FBP, FBW, PSM, DBX, DBY) \
	(((u_long)(FBP) << 0)       | ((u_long)(FBW) << 9)       |\
	 ((u_long)(PSM) << 15)      | ((u_long)(DBX) << 32)      |\
	 ((u_long)(DBY) << 43))
#define DISPFBn_FBP_M	(0x1FF<<0)
#define DISPFBn_FBW_M	(0x3F<<9)
#define DISPFBn_PSM_M	(0x1F<<15)
#define DISPFBn_DBX_M	(0x7FF<<32)
#define DISPFBn_DBY_M	(0x7FF<<43)

#define DISPFBn_FBP_O	(0)
#define DISPFBn_FBW_O	(9)
#define DISPFBn_PSM_O	(15)
#define DISPFBn_DBX_O	(32)
#define DISPFBn_DBY_O	(43)
// DISPFBn instances
#define DISPFB1 	((volatile u_long *)(0x12000070))
#define DISPFB2 	((volatile u_long *)(0x12000090))



//DISPLAYn
typedef struct {
	 u_long DX  :12;	// x position in the diaplay area (vck units)
	 u_long DY  :11;	// x position in the diaplay area (raster units)
	 u_long MAGH:4;	// horizontal magnification
	 u_long MAGV:2;	// vertical magnification
	 u_long pad0:3;	// padding
	 u_long DW  :12;	// display area width (vck units)
	 u_long DH  :11;	// display area height (pixel units)
	 u_long pad1:9;	// padding
} stDISPLAYn;
// DISPLAYn SET macro
#define LIB_SET_DISPLAY1	LIB_SET_DISPLAYn
#define LIB_SET_DISPLAY2	LIB_SET_DISPLAYn
#define LIB_SET_DISPLAYn(DX, DY, MAGH, MAGV, DW, DH) \
	(((u_long)(DX) << 0)        | ((u_long)(DY) << 12)       |\
	 ((u_long)(MAGH) << 23)     | ((u_long)(MAGV) << 27)     |\
	 ((u_long)(DW) << 32)       | ((u_long)(DH) << 44))
#define DISPLAYn_DX_M	(0xFFF<<0)
#define DISPLAYn_DY_M	(0x7FF<<12)
#define DISPLAYn_MAGH_M	(0xF<<23)
#define DISPLAYn_MAGV_M	(0x3<<27)
#define DISPLAYn_DW_M	(0xFFF<<32)
#define DISPLAYn_DH_M	(0x7FF<<44)

#define DISPLAYn_DX_O	(0)
#define DISPLAYn_DY_O	(12)
#define DISPLAYn_MAGH_O	(23)
#define DISPLAYn_MAGV_O	(27)
#define DISPLAYn_DW_O	(32)
#define DISPLAYn_DH_O	(44)
// DISPLAYn instances
#define DISPLAY1 	((volatile u_long *)(0x12000080))
#define DISPLAY2 	((volatile u_long *)(0x120000a0))



//EXTBUF
typedef struct {
	 u_long EXBP :14;	// 
	 u_long EXBW :6;	// 
	 u_long FBIN :2;	// 
	 u_long WFFMD:1;	// 
	 u_long EMODA:2;	// 
	 u_long EMODC:2;	// 
	 u_long pad0 :5;	// padding
	 u_long WDX  :11;	// 
	 u_long WDY  :11;	// 
	 u_long pad1 :10;	// padding
} stEXTBUF;
// EXTBUF SET macro
#define LIB_SET_EXTBUF(EXBP, EXBW, FBIN, WFFMD, EMODA, EMODC, WDX, WDY) \
	(((u_long)(EXBP) << 0)      | ((u_long)(EXBW) << 14)     |\
	 ((u_long)(FBIN) << 20)     | ((u_long)(WFFMD) << 22)    |\
	 ((u_long)(EMODA) << 23)    | ((u_long)(EMODC) << 25)    |\
	 ((u_long)(WDX) << 32)      | ((u_long)(WDY) << 43))
#define EXTBUF_EXBP_M	(0x3FFF<<0)
#define EXTBUF_EXBW_M	(0x3F<<14)
#define EXTBUF_FBIN_M	(0x3<<20)
#define EXTBUF_WFFMD_M	(0x1<<22)
#define EXTBUF_EMODA_M	(0x3<<23)
#define EXTBUF_EMODC_M	(0x3<<25)
#define EXTBUF_WDX_M	(0x7FF<<32)
#define EXTBUF_WDY_M	(0x7FF<<43)

#define EXTBUF_EXBP_O	(0)
#define EXTBUF_EXBW_O	(14)
#define EXTBUF_FBIN_O	(20)
#define EXTBUF_WFFMD_O	(22)
#define EXTBUF_EMODA_O	(23)
#define EXTBUF_EMODC_O	(25)
#define EXTBUF_WDX_O	(32)
#define EXTBUF_WDY_O	(43)
// EXTBUF instances
#define EXTBUF 	((volatile u_long *)(0x120000b0))



//EXTDATA
typedef struct {
	 u_long SX  :12;	// 
	 u_long SY  :11;	// 
	 u_long SMPH:4;	// 
	 u_long SMPV:2;	// 
	 u_long pad0:3;	// padding
	 u_long WW  :12;	// 
	 u_long WH  :11;	// 
	 u_long pad1:9;	// padding
} stEXTDATA;
// EXTDATA SET macro
#define LIB_SET_EXTDATA(SX, SY, SMPH, SMPV, WW, WH) \
	(((u_long)(SX) << 0)        | ((u_long)(SY) << 12)       |\
	 ((u_long)(SMPH) << 23)     | ((u_long)(SMPV) << 27)     |\
	 ((u_long)(WW) << 32)       | ((u_long)(WH) << 44))
#define EXTDATA_SX_M	(0xFFF<<0)
#define EXTDATA_SY_M	(0x7FF<<12)
#define EXTDATA_SMPH_M	(0xF<<23)
#define EXTDATA_SMPV_M	(0x3<<27)
#define EXTDATA_WW_M	(0xFFF<<32)
#define EXTDATA_WH_M	(0x7FF<<44)

#define EXTDATA_SX_O	(0)
#define EXTDATA_SY_O	(12)
#define EXTDATA_SMPH_O	(23)
#define EXTDATA_SMPV_O	(27)
#define EXTDATA_WW_O	(32)
#define EXTDATA_WH_O	(44)
// EXTDATA instances
#define EXTDATA 	((volatile u_long *)(0x120000c0))



//EXTWRITE
typedef struct {
	 u_long WRITE:1;	// activation/deactivation of write
	 u_long pad0 :63;	// padding
} stEXTWRITE;
// EXTWRITE SET macro
#define LIB_SET_EXTWRITE(WRITE) \
	(((u_long)(WRITE) << 0))
#define EXTWRITE_WRITE_M	(0x1<<0)

#define EXTWRITE_WRITE_O	(0)
// EXTWRITE instances
#define EXTWRITE 	((volatile u_long *)(0x120000d0))



//IMR
typedef struct {
	 u_long pad0     :8;	// padding
	 u_long SIGMSK   :1;	// signal event interrupt mask
	 u_long FINISHMSK:1;	// finish event interrupt mask
	 u_long HSMSK    :1;	// hsync interrupt mask
	 u_long VSMSK    :1;	// vsync interrupt mask
	 u_long EDWMSK   :1;	// rect area write termination interrupt mask
	 u_long RESERVED :2;	// reserved
	 u_long pad1     :49;	// padding
} stIMR;
// IMR SET macro
#define LIB_SET_IMR(SIGMSK, FINISHMSK, HSMSK, VSMSK, EDWMSK, RESERVED) \
	(((u_long)(SIGMSK) << 8)    | ((u_long)(FINISHMSK) << 9) |\
	 ((u_long)(HSMSK) << 10)    | ((u_long)(VSMSK) << 11)    |\
	 ((u_long)(EDWMSK) << 12)   | ((u_long)(RESERVED) << 13))
#define IMR_SIGMSK_M	(0x1<<8)
#define IMR_FINISHMSK_M	(0x1<<9)
#define IMR_HSMSK_M	(0x1<<10)
#define IMR_VSMSK_M	(0x1<<11)
#define IMR_EDWMSK_M	(0x1<<12)
#define IMR_RESERVED_M	(0x3<<13)

#define IMR_SIGMSK_O	(8)
#define IMR_FINISHMSK_O	(9)
#define IMR_HSMSK_O	(10)
#define IMR_VSMSK_O	(11)
#define IMR_EDWMSK_O	(12)
#define IMR_RESERVED_O	(13)
// IMR instances
#define IMR 	((volatile u_long *)(0x12001010))



//PMODE
typedef struct {
	 u_long EN1  :1;	// read circuit 1 on/off
	 u_long EN2  :1;	// read circuit 2 on/off
	 u_long CRTMD:3;	// crt output switching
	 u_long MMOD :1;	// alpha value selection for alpha blending
	 u_long AMOD :1;	// out1 alpha output selection
	 u_long SLBG :1;	// alpha blending method selection
	 u_long ALP  :8;	// fixed alpha value (0xff=1.0)
	 u_long pad0 :48;	// padding
} stPMODE;
// PMODE SET macro
#define LIB_SET_PMODE(EN1, EN2, CRTMD, MMOD, AMOD, SLBG, ALP) \
	(((u_long)(EN1) << 0)   | ((u_long)(EN2) << 1)   |\
	 ((u_long)(CRTMD) << 2) | ((u_long)(MMOD) << 5)  |\
	 ((u_long)(AMOD) << 6)  | ((u_long)(SLBG) << 7)  |\
	 ((u_long)(ALP) << 8))
#define PMODE_EN1_M	(0x1<<0)
#define PMODE_EN2_M	(0x1<<1)
#define PMODE_CRTMD_M	(0x7<<2)
#define PMODE_MMOD_M	(0x1<<5)
#define PMODE_AMOD_M	(0x1<<6)
#define PMODE_SLBG_M	(0x1<<7)
#define PMODE_ALP_M	(0xFF<<8)

#define PMODE_EN1_O	(0)
#define PMODE_EN2_O	(1)
#define PMODE_CRTMD_O	(2)
#define PMODE_MMOD_O	(5)
#define PMODE_AMOD_O	(6)
#define PMODE_SLBG_O	(7)
#define PMODE_ALP_O	(8)
// PMODE instances
#define PMODE 	((volatile u_long *)(0x12000000))



//SIGLBLD
typedef struct {
	 u_long SIGID:32;	// id value set by signal register
	 u_long LBLID:32;	// id value set by label register
} stSIGLBLD;
// SIGLBLD SET macro
#define LIB_SET_SIGLBLD(SIGID, LBLID) \
	(((u_long)(SIGID) << 0)    | ((u_long)(LBLID) << 32))
#define SIGLBLD_SIGID_M	(0xFFFFFFFF<<0)
#define SIGLBLD_LBLID_M	(0xFFFFFFFF<<32)

#define SIGLBLD_SIGID_O	(0)
#define SIGLBLD_LBLID_O	(32)
// SIGLBLD instances
#define SIGLBLD 	((volatile u_long *)(0x12001080))



//SMODE2
typedef struct {
	 u_long INT :1;	// interlace mode setting
	 u_long FFMD:1;	// setting in interlaced mode
	 u_long DPMS:2;	// vesa dpms mode setting
	 u_long pad0:60;	// padding
} stSMODE2;
// SMODE2 SET macro
#define LIB_SET_SMODE2(INT, FFMD, DPMS) \
	(((u_long)(INT) << 0)  | ((u_long)(FFMD) << 1) |\
	 ((u_long)(DPMS) << 2))
#define SMODE2_INT_M	(0x1<<0)
#define SMODE2_FFMD_M	(0x1<<1)
#define SMODE2_DPMS_M	(0x3<<2)

#define SMODE2_INT_O	(0)
#define SMODE2_FFMD_O	(1)
#define SMODE2_DPMS_O	(2)
// SMODE2 instances
#define SMODE2 	((volatile u_long *)(0x12000020))


#endif
//
