#ifndef __IPU_H_
#define __IPU_H_
// IPU REGISTERS structs


//IPU_CMDw
typedef struct {
	 u_long OPTION:28;	// command option
	 u_long CODE  :4;	// command code
	 u_long pad0  :32;	// padding
} stIPU_CMDw;
// IPU_CMDw SET macro
#define LIB_SET_IPU_CMDw(OPTION, CODE) \
	(((u_long)(OPTION) << 0)  | ((u_long)(CODE) << 28))
#define IPU_CMDw_OPTION_M	(0xFFFFFFF<<0)
#define IPU_CMDw_CODE_M	(0xF<<28)

#define IPU_CMDw_OPTION_O	(0)
#define IPU_CMDw_CODE_O	(28)
// IPU_CMDw instances



//IPU_CMDr
typedef struct {
	 u_long DATA:32;	// vdec/fdec decoded value
	 u_long pad0:31;	// padding
	 u_long BUSY:1;	// vdec/fdec command busy
} stIPU_CMDr;
// IPU_CMDr SET macro
#define LIB_SET_IPU_CMD	LIB_SET_IPU_CMDr
#define LIB_SET_IPU_CMDr(DATA, BUSY) \
	(((u_long)(DATA) << 0)       | ((u_long)(BUSY) << 63))
#define IPU_CMDr_DATA_M	(0xFFFFFFFF<<0)
#define IPU_CMDr_BUSY_M	(0x1<<63)

#define IPU_CMDr_DATA_O	(0)
#define IPU_CMDr_BUSY_O	(63)
// IPU_CMDr instances
#define IPU_CMD 	((volatile u_long *)(0x10003900))



//IPU_TOP
typedef struct {
	 u_long BSTOP:32;	// top 32 bits of the bit stream
	 u_long pad0 :31;	// padding
	 u_long BUSY :1;	// command busy
} stIPU_TOP;
// IPU_TOP SET macro
#define LIB_SET_IPU_TOP(BSTOP, BUSY) \
	(((u_long)(BSTOP) << 0)      | ((u_long)(BUSY) << 63))
#define IPU_TOP_BSTOP_M	(0xFFFFFFFF<<0)
#define IPU_TOP_BUSY_M	(0x1<<63)

#define IPU_TOP_BSTOP_O	(0)
#define IPU_TOP_BUSY_O	(63)
// IPU_TOP instances
#define IPU_TOP 	((volatile u_long *)(0x10003900))



//IPU_CTRL
typedef struct {
	 u_int IFC :4;	// input fifo counter
	 u_int OFC :4;	// output fifo counter
	 u_int CBP :6;	// coded block pattern
	 u_int ECD :1;	// error code detected
	 u_int SCD :1;	// start code detected
	 u_int IDP :2;	// intra dc precision
	 u_int pad0:2;	// padding
	 u_int AS  :1;	// alternate scan
	 u_int IVF :1;	// intra vlc format
	 u_int QST :1;	// q scale step
	 u_int MP1 :1;	// mpeg1 bitstream
	 u_int PCT :3;	// picture type
	 u_int pad1:3;	// padding
	 u_int RST :1;	// reset
	 u_int BUSY:1;	// busy
} stIPU_CTRL;
// IPU_CTRL SET macro
#define LIB_SET_IPU_CTRL(IFC, OFC, CBP, ECD, SCD, IDP, AS, IVF, QST, MP1, PCT, RST, BUSY) \
	(((u_long)(IFC) << 0)     | ((u_long)(OFC) << 4)     |\
	 ((u_long)(CBP) << 8)     | ((u_long)(ECD) << 14)    |\
	 ((u_long)(SCD) << 15)    | ((u_long)(IDP) << 16)    |\
	 ((u_long)(AS) << 20)     | ((u_long)(IVF) << 21)    |\
	 ((u_long)(QST) << 22)    | ((u_long)(MP1) << 23)    |\
	 ((u_long)(PCT) << 24)    | ((u_long)(RST) << 30)    |\
	 ((u_long)(BUSY) << 31))
#define IPU_CTRL_IFC_M	(0xF<<0)
#define IPU_CTRL_OFC_M	(0xF<<4)
#define IPU_CTRL_CBP_M	(0x3F<<8)
#define IPU_CTRL_ECD_M	(0x1<<14)
#define IPU_CTRL_SCD_M	(0x1<<15)
#define IPU_CTRL_IDP_M	(0x3<<16)
#define IPU_CTRL_AS_M	(0x1<<20)
#define IPU_CTRL_IVF_M	(0x1<<21)
#define IPU_CTRL_QST_M	(0x1<<22)
#define IPU_CTRL_MP1_M	(0x1<<23)
#define IPU_CTRL_PCT_M	(0x7<<24)
#define IPU_CTRL_RST_M	(0x1<<30)
#define IPU_CTRL_BUSY_M	(0x1<<31)

#define IPU_CTRL_IFC_O	(0)
#define IPU_CTRL_OFC_O	(4)
#define IPU_CTRL_CBP_O	(8)
#define IPU_CTRL_ECD_O	(14)
#define IPU_CTRL_SCD_O	(15)
#define IPU_CTRL_IDP_O	(16)
#define IPU_CTRL_AS_O	(20)
#define IPU_CTRL_IVF_O	(21)
#define IPU_CTRL_QST_O	(22)
#define IPU_CTRL_MP1_O	(23)
#define IPU_CTRL_PCT_O	(24)
#define IPU_CTRL_RST_O	(30)
#define IPU_CTRL_BUSY_O	(31)
// IPU_CTRL instances
#define IPU_CTRL 	((volatile u_int *)(0x10003900))



//IPU_BP
typedef struct {
	 u_int BP  :7;	// bit stream pointer
	 u_int pad0:1;	// padding
	 u_int IFC :4;	// input fifo counter
	 u_int pad1:4;	// padding
	 u_int FP  :2;	// fifo pointer
	 u_int pad2:14;	// padding
} stIPU_BP;
// IPU_BP SET macro
#define LIB_SET_IPU_BP(BP, IFC, FP) \
	(((u_long)(BP) << 0)    | ((u_long)(IFC) << 8)   |\
	 ((u_long)(FP) << 16))
#define IPU_BP_BP_M	(0x7F<<0)
#define IPU_BP_IFC_M	(0xF<<8)
#define IPU_BP_FP_M	(0x3<<16)

#define IPU_BP_BP_O	(0)
#define IPU_BP_IFC_O	(8)
#define IPU_BP_FP_O	(16)
// IPU_BP instances
#define IPU_BP 	((volatile u_int *)(0x10003900))


#endif
//
