#ifndef __TIMER_H_
#define __TIMER_H_
// TIMER REGISTERS structs


//Tn_MODE
typedef struct {
	 u_int CLKS:2;	// clock selection
	 u_int GATE:1;	// gate function enable
	 u_int GATS:1;	// gate selection
	 u_int GATM:2;	// gate mode
	 u_int ZRET:1;	// zero return
	 u_int CUE :1;	// count up enable
	 u_int CMPE:1;	// compare-interrupt enable
	 u_int OVFE:1;	// overflow-interrupt enable
	 u_int EQUF:1;	// equal flag
	 u_int OVFF:1;	// overflow flag
	 u_int pad0:20;	// padding
} stTn_MODE;
// Tn_MODE SET macro
#define LIB_SET_T0_MODE	LIB_SET_Tn_MODE
#define LIB_SET_T1_MODE	LIB_SET_Tn_MODE
#define LIB_SET_T2_MODE	LIB_SET_Tn_MODE
#define LIB_SET_T3_MODE	LIB_SET_Tn_MODE
#define LIB_SET_Tn_MODE(CLKS, GATE, GATS, GATM, ZRET, CUE, CMPE, OVFE, EQUF, OVFF) \
	(((u_long)(CLKS) << 0)  | ((u_long)(GATE) << 2)  |\
	 ((u_long)(GATS) << 3)  | ((u_long)(GATM) << 4)  |\
	 ((u_long)(ZRET) << 6)  | ((u_long)(CUE) << 7)   |\
	 ((u_long)(CMPE) << 8)  | ((u_long)(OVFE) << 9)  |\
	 ((u_long)(EQUF) << 10) | ((u_long)(OVFF) << 11))
#define Tn_MODE_CLKS_M	(0x3<<0)
#define Tn_MODE_GATE_M	(0x1<<2)
#define Tn_MODE_GATS_M	(0x1<<3)
#define Tn_MODE_GATM_M	(0x3<<4)
#define Tn_MODE_ZRET_M	(0x1<<6)
#define Tn_MODE_CUE_M	(0x1<<7)
#define Tn_MODE_CMPE_M	(0x1<<8)
#define Tn_MODE_OVFE_M	(0x1<<9)
#define Tn_MODE_EQUF_M	(0x1<<10)
#define Tn_MODE_OVFF_M	(0x1<<11)

#define Tn_MODE_CLKS_O	(0)
#define Tn_MODE_GATE_O	(2)
#define Tn_MODE_GATS_O	(3)
#define Tn_MODE_GATM_O	(4)
#define Tn_MODE_ZRET_O	(6)
#define Tn_MODE_CUE_O	(7)
#define Tn_MODE_CMPE_O	(8)
#define Tn_MODE_OVFE_O	(9)
#define Tn_MODE_EQUF_O	(10)
#define Tn_MODE_OVFF_O	(11)
// Tn_MODE instances
#define T0_MODE 	((volatile u_int *)(0x10000010))
#define T1_MODE 	((volatile u_int *)(0x10000810))
#define T2_MODE 	((volatile u_int *)(0x10001010))
#define T3_MODE 	((volatile u_int *)(0x10001810))



//Tn_COUNT
typedef struct {
	 u_int COUNT:16;	// counter value
	 u_int pad0 :16;	// padding
} stTn_COUNT;
// Tn_COUNT SET macro
#define LIB_SET_T0_COUNT	LIB_SET_Tn_COUNT
#define LIB_SET_T1_COUNT	LIB_SET_Tn_COUNT
#define LIB_SET_T2_COUNT	LIB_SET_Tn_COUNT
#define LIB_SET_T3_COUNT	LIB_SET_Tn_COUNT
#define LIB_SET_Tn_COUNT(COUNT) \
	(((u_long)(COUNT) << 0))
#define Tn_COUNT_COUNT_M	(0xFFFF<<0)

#define Tn_COUNT_COUNT_O	(0)
// Tn_COUNT instances
#define T0_COUNT 	((volatile u_int *)(0x10000000))
#define T1_COUNT 	((volatile u_int *)(0x10000800))
#define T2_COUNT 	((volatile u_int *)(0x10001000))
#define T3_COUNT 	((volatile u_int *)(0x10001800))



//Tn_COMP
typedef struct {
	 u_int COMP:16;	// compare value
	 u_int pad0:16;	// padding
} stTn_COMP;
// Tn_COMP SET macro
#define LIB_SET_T0_COMP	LIB_SET_Tn_COMP
#define LIB_SET_T1_COMP	LIB_SET_Tn_COMP
#define LIB_SET_T2_COMP	LIB_SET_Tn_COMP
#define LIB_SET_T3_COMP	LIB_SET_Tn_COMP
#define LIB_SET_Tn_COMP(COMP) \
	(((u_long)(COMP) << 0))
#define Tn_COMP_COMP_M	(0xFFFF<<0)

#define Tn_COMP_COMP_O	(0)
// Tn_COMP instances
#define T0_COMP 	((volatile u_int *)(0x10000020))
#define T1_COMP 	((volatile u_int *)(0x10000820))
#define T2_COMP 	((volatile u_int *)(0x10001020))
#define T3_COMP 	((volatile u_int *)(0x10001820))



//Tn_HOLD
typedef struct {
	 u_int HOLD:16;	// hold value
	 u_int pad0:16;	// padding
} stTn_HOLD;
// Tn_HOLD SET macro
#define LIB_SET_T1_HOLD	LIB_SET_Tn_HOLD
#define LIB_SET_T2_HOLD	LIB_SET_Tn_HOLD
#define LIB_SET_Tn_HOLD(HOLD) \
	(((u_long)(HOLD) << 0))
#define Tn_HOLD_HOLD_M	(0xFFFF<<0)

#define Tn_HOLD_HOLD_O	(0)
// Tn_HOLD instances
#define T1_HOLD 	((volatile u_int *)(0x10000030))
#define T2_HOLD 	((volatile u_int *)(0x10000830))


#endif
//
