
// audiolib.h 2.0,  (c) Hermes/PS2REALITY 2003-2005 
// USO COMERCIAL PROHIBIDO! PROHIBITED USE COMMERCIAL!

#if !defined(_AUDIOLIB_)
#define _AUDIOLIB_

////////////////////////////////////////////////////////////////////////////
// variables internas
////////////////////////////////////////////////////////////////////////////
typedef struct
{
unsigned dirSPU2;
unsigned size;
} TABSAMPLES;

// tabsamples: tabla donde se aloja el comienzo y el tamao de un VAG
// esta tabla es enviada al IOP cuando se alojan los sonidos
extern TABSAMPLES tabsamples[256]; // 256 samples maximo


// conversion PCM a VAG
// pcm2vag(vag,wave,sample_len)
// vag-> puntero donde se almacenara el VAG 
// wave-> puntero de una onda PCM 16 bits MONO
// sample_len-> longitud en bytes de la onda PCM
// ret-> size del VAG
extern int pcm2vag(u8 *vag, short *wave, int sample_len);

// DMAtoIOP(src,dst,size)
// envia datos al IOP usando DMA
// src-> puntero al EE (alineado a 16 bytes)
// dst-> direccion en IOP (alineado a 4 bytes)
// size-> tamao en bytes
int DMAtoIOP(void *src,u32 dst,int size);

/*****************************************************************************************/
/* SONIDO GENERAL                                                                        */
/*****************************************************************************************/
// SoundInit()
// inicializacion de la RPC en IOP
int SoundInit(void);

// SoundAllocIOP : Asigna memoria del IOP
// size-> tamao en bytes
// ret-> direccion de memoria en IOP
unsigned  SoundAllocIOP(unsigned size);
// SoundFreeIOP : Libera memoria del IOP
// punt-> Direccion en IOP
void  SoundFreeIOP(unsigned punt);

/*****************************************************************************************/
/* MANEJO DE VOCES                                                                       */
/*****************************************************************************************/

/**  asignacion de VAG **/

// SPU2malloc (interno)
// aloja bloques de memoria del SPU2.
// size= tamao en bytes a alojar
// retorno-> ==0 no se pudo alojar; !=0 direccion de inicio del bloque en SPU2 
unsigned SPU2malloc(int size);

// SPU2freeAll()
// borra el bitmap de asignacion de bloques de memoria

void SPU2freeAll();

// SPU2free(punt) (interno)
// libera bloque de memoria del SPU2 asignado a ese puntero

void SPU2free(unsigned punt);

// SoundReleaseAllVag()
// libera todos los VAG del montculo del EE

void SoundReleaseAllVag();

// SoundReleaseVag(sample)
// libera el VAG del montculo del EE

void SoundReleaseVag(int sample);

// SoundLoadVag(sample,name)
// Carga un fichero Vag desde un dispositivo
// sample-> 0 a 255; Especifica el ndice donde se guardara el Sample
// name-> ruta completa donde encontrar el fichero
// ret-> ==0 Sin errores !=0 fallo
int SoundLoadVag(int sample,char *name);

// SoundGetCabVag(cab,blong,bfrec,btimems)
// Devuelve los valores utiles de la cabecera de un VAG (primeros 48 bytes)
// cab-> puntero al primer byte de cabecera. Si NULL, devuelve los datos del 
// ultimo SoundLoadVag()
// blong-> puntero que recibe la longitud de la parte VAG (despues de la cabecera)
// bfrec-> puntero que recibe la frecuencia de reproduccion de estos samples
// btimems->puntero que contiene 0 o el tiempo en ms que dura el VAG a su frecuencia

void SoundGetCabVag(void *cab,unsigned *blong,unsigned *bfrec,unsigned *btimems);

// SoundLoadMemVag(sample,punt,size)
// Carga un fichero Vag desde memoria (del EE) SIN los 48 bytes de cabecera
// sample-> 0 a 255; Especifica el ndice donde se guardara el Sample
// punt-> direccion de memoria del Vag
// size-> tamao en bytes
// ret-> ==0 Sin errores !=0 fallo
int SoundLoadMemVag(int sample,char *punt,int size);
///////////////////////////////////////////////////////////////////////////////////////////

/** control de voces **/

// SoundLoop()
// Se encarga de enviar las voces (PlayVoice) y devuelve si hay alguna en uso o no
// ret->==0 voces en uso. !=0 no hay voces en uso
int SoundLoop();


// PlayVoice(sample,frec,delaytime,time,voll,volr,ADSR)
// Reproduce un VAG en el IOP. (asignacion automatica de canal)
// sample-> ndice al sample (0 a 255)
// frec-> frecuencia central en Herzios del Vag
// delaytime-> tiempo en ms de retardo antes de ejecutar el Vag (pasos de 10ms)
// time-> tiempo en ms de duracion del Vag (pasos de 10ms)
// voll,vollr-> volumen canal izquierdo y derecho. rango : 0 a 0x7fff
// ADSR ->Efecto de audio, 0=sin efectos
//  BIT 31-29 Sustain rate mode. Modes: 000->Linear Increase, 010 Linear Decrease, 
//                              100->Pseudo Exp Increase, 110 Exponential Decrease
//  BIT 28-22 Sustain Rate
//  BIT 21    Release Rate Mode 0-> Linear Decrease Mode, 1 Exponential Decrease Mode
//  BIT 20-16 Release Rate
//  BIT 15    Attack Rate Mode 0-> Linear Increase, 1 Pseudo Exponential Increase
//  BIT 14-8  Attack Rate
//  BIT 7-4   Decay Rate
//  BIT 3-0   Sustain Level
//
// ret-> ==0 Sin Error !=0 Error

int PlayVoice(unsigned sample,int frec,int delaytime,int time,unsigned voll,unsigned volr, unsigned ADSR);



// Note2Frec(note,notacentral,frecsample)
// util de conversion nota-frecuencia
// note-> nota a convertir
// notacentral-> Nota a la que suena el Vag
// frecsample-> frecuencia de sampleo del Vag
// ret-> frecuencia calculada para note
// Ejemplo ilustrativo: Vag sampleado a 22050HZ que contiene una nota DO3 de un instrumento
// y queremos averiguar la nota DO4 -> Note2Frec(48,36,22050)
// codificacion de notas: 0->DO0 12->DO1 36->DO3
// DO,DO#,RE,RE#,MI,FA,FA#,SOL,SOL#,LA,LA#,SI -> de 0 a 11

unsigned Note2Frec(unsigned note,unsigned notacentral,unsigned frecsample);


// GetVoiceRef() -> devuelve la referencia de la ultima voz programada con PlayVoice()
// ret -> referencia 
unsigned GetVoiceRef();
// VoiceRefOff(ref)-> Apaga la voz con esa referencia
// ref -> referencia obtenida despues de programar la voz con GetVoiceRef()
void VoiceRefOff(unsigned ref);

// SoundOff() ->Apaga todas las voces (10ms de delay)
void SoundOff();

//////////////////////////////////////////////////////////////////////////////////////////

/*****************************************************************************************/
/* CONTROL PCM                                                                           */
/*****************************************************************************************/

// PCMInit(frec,channels)
// asigna parametros de inicio
// frec-> frecuencia en Hz del sonido PCM
// channels-> numero de canales: 1 o 2 (stereo)
// pone timer a 0
void PCMInit(unsigned frec,int channels);


// PCMSilence() 
// entra en un bucle de espera, hasta detectar la parada del sonido
void PCMSilence();

// PCMGetTime()
// ret-> devuelve el ultimo valor devuelto del timer de sonido
// ATENCION: requiere antes una llamada a la funcion PCMSend(), pues no es RPC
// se puede usar un PCMSend(0,0); para actualizar el contador
unsigned PCMGetTime();

// PCMTime(time)
// ajusta el contador de tiempo del sonido PCM 
// time->tiempo en ms
void PCMTime(unsigned time);

// PCMVolume(voll,volr)
// ajusta volumen izquierdo y derecho del sonido PCM 
// voll,volr->volumen de 0 a 0x7fff

void PCMVolume(unsigned voll,unsigned volr);

//PCMSend(aud,samples)
// envia samples de audio al IOP
// aud->puntero al EE donde estan almacenados los samples
// samples-> numero de samples almacenados en aud
// ret-> <0 numero de samples insuficientes para enviar al IOP
//       >0 numero de samples enviados
//       ==0 IOP ocupado
int PCMSend(unsigned short *aud,int samples);

// notas de PCMSend:
// El numero de samples maximos a enviar, sera para 2 canales, 48KHZ 16bit=
// (2*48000)*80/1000 =7680 samples. La constante 80/1000, sale del loop de audio
// (80 ms). As pues, si queremos estar seguro de que al enviar los samples, estos seran
// suficientes, enviar como minimo 7680. El buffer del IOP puede alojar 
// hasta 400ms de audio


/*****************************************************************************************/
/* CD Audio                                                                              */
/*****************************************************************************************/

#define PREGAP 150


int  CDDAGetToc(char *punt); // lectura del TOC (no precisa CDDASync()) punt debe ser un area de 2352 bytes
int  CDDAGetTracks(unsigned *track); // lee el toc y devuelve en el puntero una lista de LBA de cada pista de audio
                                    // retorna numero de pistas encontradas
void CDDA_LBAtoTIME(unsigned lba,int *min,int *sec,int *frame); // convierte un LBA a tiempo
int  CDDAReadSect(unsigned lba,unsigned nsect,char *punt); // lectura de CDAUDIO (capacidad para 12 sectores)
int  CDDASync(); // se usa en combinacion con CDDAReadSect (lectura asincrona);
int  CDDAStop(); // para el CD
int  CDDABreak(); // cancela el comando de CDROM
int  CDDATray(int param,unsigned *chktray); // controla la bandeja
int  CDDAGetError(); // cancela el comando de CDROM


//////////////////////////////////////////////////////////////////////
// Async commands (lectura mediante hilo paralelo en IOP)
//////////////////////////////////////////////////////////////////////

#define ASYNC_ERROR -1 // error en funcion
#define ASYNC_PROG  -2 // indica que la lectura se program correctamente
#define ASYNC_BUSY  -3 // indica que no es posible programar una nueva lectura por el momento

// Async_open(name,ch)
// abre un fichero para ser leido de forma asincrona (max 4)
// ch-> canal de lectura (0 a 3)
// ret-> retorna numero de canal enviado en ch (0 a 3)  , ASYNC_ERROR si hubo error


int  Async_open(char * name,int ch);

// Async_size(ch)
// retorna el tamao total del fichero abierto en ese canal (0 a 3) o ASYNC_ERROR si hubo error

int  Async_size(int ch);

// Async_read(ch,offset)
// programa la lectura de un bloque de 64KB de forma asincrona
// ch-> canal del fichero abierto (0 a 3)
// offset-> posicion en el fichero desde done se intentaran leer  65536 bytes
// *EEbuffer-> puntero que contiene la direccion del buffer de datos (64KB). Atencion: el buffer que pasamos
//             no tiene porque ser el mismo que recibimos, ya que podria haber sido cambiado en una 
//             lectura anterior. Debe estar alineado a 16 bytes
// ret-> ASYNC_ERROR : error, no es posible programa la lectura
//       ASYNC_PROG : lectura programada
//       ASYNC_BUSY : lectura ocupada
//       >=0  : Bytes leidos en lectura previa (IMPORTANTE:solo se devuelve una vez este dato)
// NOTA: si el offset es <0,no se programa lectura y solo devuelve el estado o bytes leidos anteriormente
// necesita de Async_dma() para mover los bytes desde IOP a EE

int  Async_read(int ch,int offset,unsigned *EEbuffer);

// Async_cancel(ch)
// Cancela una lectura en caso de estar programada o el resultado de esta.
// ch -> numero de canal a cancelar (0 a 3)

int  Async_cancel(int ch);

// Async_close(ch)
// cierra el fichero asincrono
// ch-> numero de canal asociado (0 a 3)
// ret-> 0: Ok, ASYNC_ERROR: error al cerrar (puede indicar que un fichero esta leyendose)
int  Async_close(int ch);

// Async_wait()
// espera hasta que se terminan todas las lecturas programadas
int  Async_wait();



#endif

