// audiolibmp3.h 1.0,  (c) Hermes/PS2REALITY 2005
// USO COMERCIAL PROHIBIDO! PROHIBITED USE COMMERCIAL!
#if !defined(_AUDIOLIBMP3_)
#define _AUDIOLIBMP3_

typedef struct
{
int bitrate;
int samplerate;
int channels;
int size;
}
infoMP3;

typedef struct 
{
unsigned year;
unsigned char track;
char title[30];
char author[30];
char album[30];
char genre[30];
char comment[30];
}
id3data;

/***************************************************************/
/** MISCELANEA                                                **/
/***************************************************************/

// id3_create_tag(id3,buf)
// crea la informacion necesaria para ID3 (de uso en encoders)
// id3-> estructura con los datos ID3 a crear
// buf-> buffer de 128 bytes donde se compone la informacion
// NOTA: la informacion ID3 es un bloque de 128 bytes situados al final de un MP3

extern void id3_create_tag(id3data *id3, unsigned char *buf); // to declare this function

/***************************************************************/
/** inicializacion                                            **/
/***************************************************************/

// InitMP3(quality)
// Inicializa la lib de MP3.
// quality-> 0: MP3 Codec rpido, 1: MP3 Codec de calidad alta
// ret-> 0: Ok -1: fallo al registrar el codec

int InitMP3(int quality); 

/***************************************************************/
/** apertura/cierre de ficheros                               **/
/***************************************************************/

// OpenMP3(name)
// abre un fichero mp2/mp3 desde un device
// name: ruta del fichero. Ejemplo: "host:/song.mp3"
// ret-> numero de canal de lectura (0 a 1) , si <0 error

int OpenMP3(char *name); // desde device: host, cdrom, mass....

// MemOpenMP3(addr,size)
// abre un fichero mp2/mp3 desde memoria
// addr: direccion en el EE donde se encuentra el fichero
// size: tamao del fichero
// ret-> numero de canal de lectura (0 a 1) , si <0 error

int MemOpenMP3(void *addr, int size); // desde memoria (EE)

// CloseMP3(ch)
// cierra el fichero asociado a ese canal de lectura
// ch: numero de canal (de 0 a 1)
// ret-> 0: ok -1: error

int CloseMP3(int ch);

// ID3

// GetID3TAG(ch, id3)
// lee la informacion ID3 desde el canal especificado
// ch: numero de canal (de 0 a 1)
// id3 estructura que recibe los datos
// ret-> 0: Ok, otro resultado: error
// NOTA: lo normal es leer esta info despues de un OpenMP3()

int GetID3TAG(int ch, id3data *id3); 

/***************************************************************/
/** caractersticas de ejecucion                              **/
/***************************************************************/

// RepeatMP3(ch,onoff);
// fija o anula flag de repeticion de MP3
// ch: numero de canal (de 0 a 1)
// onoff-> 0: anula la autorepeticion 1: activa la autorepeticion

void RepeatMP3(int ch, int onoff); 

/***************************************************************/
/** atributo de fin de fichero                                **/
/***************************************************************/

// EofMP3(ch)
// devuelve si se alcanz el final de fichero
// ch: numero de canal (de 0 a 1)
// ret: 1 (EOF) o 0

int EofMP3(int ch); // si 1 el MP3 lleg al final


/***************************************************************/
/* funciones de transicion entre MP3's                         */
/***************************************************************/

// FastChangeMP3(ch)
// cambio rapido de reproduccion al canal especificado (con perdida de samples)
// ch: numero de canal (de 0 a 1)

int FastChangeMP3(int ch); 

// ChangeMP3(ch)
// cambio normal de reproduccion al canal especificado (no pierde samples)
// ch: numero de canal (de 0 a 1)

int ChangeMP3(int ch); 

/***************************************************************/
// funciones para MP3 en ejecucion
/***************************************************************/

// GetMP3ch()
// ret->retorna canal en reproduccion

int GetMP3ch(); 

// GetTimeMP3()
// ret-> retorna tiempo en segundos desde el inicio

int GetTimeMP3(); 


// SkipTimeMP3(sec)
// salta los segundos especificados desde el inicio del fichero
// sec: numero de segundos. Si se salta relativamente, se recomienda saltar al menos 5 segundos
// ret: 0: Ok

int SkipTimeMP3(unsigned sec); // salta sec segundos desde el inicio

// ResetMP3()
// reinicia el MP3

int ResetMP3();

// ReadAndDecodeMP3()
// funcion de lectura asincrona, descodificacin y preparacin de samples: todo en uno

void ReadAndDecodeMP3(); 

// SendSamplesMP3()
// funcion de envio de samples hacia IOP. Junto con ReadAndDecodeMP3, conforma el MP3 player

void SendSamplesMP3(); 

// WaitFileMP3()
// espera a finalizar la ultima lectura programada.
// tiene su utilidad cuando se asigna un canal para reproducir y queremos asegurar que antes de
// reproducir, se haya completado una lectura desde el stream

void WaitFileMP3();  

// GetInfoMP3(i)
// obtiene informacion relativa al numero de canales, bitrate, samplerate(Hz) y tamao del fichero
// en reproduccion
// i: estruc tura que recibe la informacion
// ret: 0: OK, otro valor: informacion no disponible
// NOTA: conviene llamarla despues de ReadAndDecodeMP3()

int GetInfoMP3(infoMP3 *i); 

// GetSamplesMP3(samples)
// Obtiene la direccion interna de los samples enviados al IOP
// samples: direccion de un puntero (short) que recibe el buffer interno. Si recibe 0, ignorar
// ret: numero de samples que contiene el buffer (correspondiente a los dos canales o a uno)
// NOTA: esta funcion debe ser llamada DESPUES de SendSamplesMP3() y solo devuelve una vez el 
// numero de samples. Tiene su interes para visualizar graficamente la onda

int GetSamplesMP3(short **samples);
 
/***************************************************************/								
// encode: estas funciones, realmente, generan MP2
/***************************************************************/

/** NOTA: ver cdda2 example **/

// EncodeInitMP3(bitrate,samplerate,channels)
// inicializa el encoder
// bitrate: numero de bits por segundo para la codificacion
// samplerate: frecuencia en Hz de los samples a codificar
// channels: numero de canales ( 1 o 2). En caso de ser 2, los samples de izq/der deben ir alternados
// ret-> 0: Ok

int EncodeInitMP3(int bitrate,int samplerate,int channels); // inicializa el encoder

// EncodeMP3(samples,nsamples,outbuf)
// codifica una serie de samples a MP2
// samples: puntero del buffer que contiene los samples a codificar
// nsamples: puntero de entrada/salida. De entrada recibe los samples que contiene el buffer samples.
//           de salida, contiene el numero de samples gastados en la codificacion
// outbuf: contiene el buffer donde se almacena la codificacion (debe ser de 16384 bytes)
// ret-> <0: error 0: no se produjo codificacion >0 numero de bytes en outbuf para escribir

int EncodeMP3(unsigned short *samples,int *nsamples,void *outbuf); // obtiene un stream mp2

// EncodeEndMP3()
// finaliza el encoder

int EncodeEndMP3(); 

#endif

