/***************************************************************
/  Ifcaro3D - Preview                                          /
/                                                              /
/  Creado por Ifcaro <taslokooo@gmail.com>                     /
/  http://ps2dev.ifcaro.net                                    /
/                                                              /
/  Contiene codigo de ps2game:                                 /
/  Angel Garbayo Dominguez (neofar@ofcode.com)                 /
/  http://ps2dev.ofcode.com                                    /
/                                                              /
/  last mod: 20-11-2008                                        /
/  license:  GNU Library General Public License Version 2      /
/            (see LICENSE on the main directory)               /
***************************************************************/

#include "Ifcaro3D.h"
#include "entities.h"

PtrNodoObj objPtr = NULL; 
PtrNodoLuz luzPtr = NULL; 

// Graficos

void RenderWorld(){
	renderList(objPtr);
	camRender(actualCamera);
}

// Entidad: Creacion

TEntity CreateCamera(){
	Tentity* entity=(Tentity *)malloc(sizeof(Tentity));
	entity->escamara=cameraCreate();
	entity->clase=TENTITY_CAMARA;
	free(entity->esobjeto);
	//free(entity->ode);
	free(entity->esluz);
	return entity;
}

TEntity CreateLight(){
	Tentity* entity=(Tentity *)malloc(sizeof(Tentity));
	entity->esluz = lightCreate(0,0,0);
	entity->clase=TENTITY_LUZ;
	insLuz( &luzPtr, entity );

	asignarluz2(luzPtr, objPtr);
	
	free(entity->escamara);
	free(entity->esobjeto);
	//free(entity->ode);
	return entity;
}

TEntity LoadMesh(char* archivo, ...){

va_list args;
int ode;
va_start(args,archivo);
ode=va_arg(args, int);
va_end(args);

	char*ext=minusculas(substr(archivo,strlen(archivo)-3,strlen(archivo)));
	if(!strcmp(ext,"3dm")){
		Model3DM *modelo=(Model3DM*)malloc(sizeof(Model3DM));
		memset (modelo,0,sizeof(modelo));
		Tentity* entity=(Tentity *)malloc(sizeof(Tentity));
		memset (entity,0,sizeof(entity));
		entity->clase=TENTITY_OBJETO;

		modelo = LoadModel3DM(archivo);
		entity->esobjeto=ObjectCreate(modelo);
		objectSetRenderMode(entity->esobjeto, RENDER_FLAT);
		//objectSetColor(entity->esobjeto,0xFF,0xFF,0xFF);
	
		entity->rotacion[0]=0;
		entity->rotacion[1]=0;
		entity->rotacion[2]=0;
				
		entity->solid=ode;
		
		if (entity->solid==1 || entity->solid==2){
			int n=0,x1=0,y1=0,z1=0,x2=0,y2=0,z2=0;
			for(n=0;n<entity->esobjeto->generator->NUMVERTEX;n++){
				if((int)entity->esobjeto->generator->vertices[n][0]<x1){
					x1=entity->esobjeto->generator->vertices[n][0];
				}
				if((int)entity->esobjeto->generator->vertices[n][0]>x2){
					x2=entity->esobjeto->generator->vertices[n][0];
				}
				if((int)entity->esobjeto->generator->vertices[n][1]<y1){
					y1=entity->esobjeto->generator->vertices[n][1];
				}
				if((int)entity->esobjeto->generator->vertices[n][1]>y2){
					y2=entity->esobjeto->generator->vertices[n][1];
				}
				if((int)entity->esobjeto->generator->vertices[n][2]<z1){
					z1=entity->esobjeto->generator->vertices[n][2];
				}
				if((int)entity->esobjeto->generator->vertices[n][2]>z2){
					z2=entity->esobjeto->generator->vertices[n][2];
				}
			}
			if(entity->solid==1){
				entity->ode=ODE_crearcubo(0,0,0,x2-x1,y2-y1,z2-z1);
				//entity->ode=ODE_creartrimesh(0,0,0,x2-x1,y2-y1,z2-z1,entity->esobjeto->generator);
			}else if(entity->solid==2){
				int radio=x2-x1;
				if(y2-y1>x2-x1) radio= y2-y1;
				if(z2-z1>x2-x1) radio= z2-z1;
				if(z2-z1>y2-y1) radio= z2-z1;
				entity->ode=ODE_crearesfera(0,0,0,radio/2);
			}
			
		}
		
		insObject(&objPtr, entity);

		
		if (luzPtr!=NULL){
			asignarluz(luzPtr, entity->esobjeto);
		}
		
		
		free(entity->escamara);
		free(entity->esluz);
		return entity;
	}
	return NULL;
}

void FreeEntity(TEntity entity){
	if(entity->clase==TENTITY_CAMARA){
		free(entity->escamara);
		free(entity);
	}else if(entity->clase==TENTITY_OBJETO){
		borrarObject(&objPtr,entity->id);
		dGeomDestroy(entity->ode.geom);
		dBodyDestroy(entity->ode.body);

		objectFree(entity->esobjeto);
		free(entity);
	}else if(entity->clase==TENTITY_LUZ){
		borrarLuz(&luzPtr,entity->id);
		lightSetActive(entity->esluz,0);
		lightReset(entity->esluz);
		lightFree(entity->esluz);
		free(entity);
	}
}

// Entidad: Control

/*void SetEntityMass(TEntity entity, int mass){
	dMass m;
	dMassSetBoxTotal (&m,mass,1,1,1);
	dBodySetMass (entity->ode.body,&m);
}*/

void PositionEntity(TEntity entity, float x, float y, float z){
	if(entity->clase==TENTITY_CAMARA){
		camPosition(entity->escamara,x,y,z);
	}else if(entity->clase==TENTITY_OBJETO){
		if(entity->solid==1 || entity->solid==2){
			ODE_moverobjeto(entity->ode, -x,y,z);
		}else{
			objectTranslate(entity->esobjeto,x,y,z);
		}
	}else if(entity->clase==TENTITY_LUZ){
		lightTranslate(entity->esluz,x,y,z);
	}
}

void MoveEntity(TEntity entity, float x, float y, float z){
	if(entity->clase==TENTITY_CAMARA){
		camMove(entity->escamara,x,y,z);
	}else if(entity->clase==TENTITY_OBJETO){
		if(entity->solid==1 || entity->solid==2){
			dBodyAddRelForce(entity->ode.body,x,y,z);
		}else{
			objectMove(entity->esobjeto,x,y,z);
		}
	}
}

void RotateEntity(TEntity entity, float x, float y, float z){
	if(entity->clase==TENTITY_CAMARA){
		camRotate(entity->escamara,gradosaradianes(x),gradosaradianes(y),gradosaradianes(z));
	}else if(entity->clase==TENTITY_OBJETO){
		if(entity->solid==1 || entity->solid==2){
			ODE_rotarobjeto(entity->ode, gradosaradianes(x), gradosaradianes(y), gradosaradianes(z));
		}else{
			objectRotate(entity->esobjeto,gradosaradianes(x),gradosaradianes(y),gradosaradianes(z));
		}
	}
}

void TurnEntity(TEntity entity, float x, float y, float z){
	if(entity->clase==TENTITY_CAMARA){
		camTurn(entity->escamara,gradosaradianes(x),gradosaradianes(y),gradosaradianes(z));
	}else if(entity->clase==TENTITY_OBJETO){
		if(entity->solid==1 || entity->solid==2){
			ODE_girarobjeto(entity->ode, gradosaradianes(x), gradosaradianes(y), gradosaradianes(z));
		}else{
			objectTurn(entity->esobjeto,gradosaradianes(x),gradosaradianes(y),gradosaradianes(z));
		//ODE_girarobjeto(entity->ode, gradosaradianes(x), gradosaradianes(y), gradosaradianes(z));
		}
	}
}

void EntityColor(TEntity entity, unsigned char r, unsigned char g, unsigned char b){
	objectSetColor(entity->esobjeto, r, g, b);
}

void EntityAlpha(TEntity entity, float alpha){
	objectSetAlpha(entity->esobjeto, (unsigned char)(255*alpha));
}

// Entidad: Estado

int EntityX(TEntity entity){
	if(entity->clase==TENTITY_CAMARA){
		return ((int)entity->escamara->posicion[0]);
	}else if(entity->clase==TENTITY_OBJETO){
		return ((int)entity->esobjeto->translation[0][0]);
	}else if(entity->clase==TENTITY_LUZ){
		return ((int)entity->esluz->posicion[0]);
	}
	return 31337;
}

int EntityY(TEntity entity){
	if(entity->clase==TENTITY_CAMARA){
		return ((int)entity->escamara->posicion[1]);
	}else if(entity->clase==TENTITY_OBJETO){
		return ((int)entity->esobjeto->translation[0][1]);
	}else if(entity->clase==TENTITY_LUZ){
		return ((int)-entity->esluz->posicion[1]);
	}
	return 31337;
}

int EntityZ(TEntity entity){
	if(entity->clase==TENTITY_CAMARA){
		return ((int)entity->escamara->posicion[2]);
	}else if(entity->clase==TENTITY_OBJETO){
		return ((int)entity->esobjeto->translation[0][2]);
	}else if(entity->clase==TENTITY_LUZ){
		return ((int)-entity->esluz->posicion[2]+1);
	}
	return 31337;
}

int EntityPitch(TEntity entity){
	if(entity->clase==TENTITY_CAMARA){
		return radianesagrados(entity->escamara->rotacion[0]);
	}else if(entity->clase==TENTITY_OBJETO){
		dReal *R = (dReal *) dBodyGetRotation(entity->ode.body);
				float x  = -atan2f(R[9], R[10]);	//phi
			while(x > PI) x -= PI_2;
		    while(x < -PI) x += PI_2;
		return radianesagrados(x);
	}
	return 31337;
}

int EntityYaw(TEntity entity){
	if(entity->clase==TENTITY_CAMARA){
		return radianesagrados(entity->escamara->rotacion[1]);
	}else if(entity->clase==TENTITY_OBJETO){
		dReal *R = (dReal *) dBodyGetRotation(entity->ode.body);
				float y = -asinf(-R[8]);		//theta
			while(y > PI) y -= PI_2;
		    while(y < -PI) y += PI_2;

		return radianesagrados(y);
	}
	return 31337;
}

int EntityRoll(TEntity entity){
	if(entity->clase==TENTITY_CAMARA){
		return radianesagrados(entity->escamara->rotacion[2]);
	}else if(entity->clase==TENTITY_OBJETO){
		dReal *R = (dReal *) dBodyGetRotation(entity->ode.body);
				float z   = -atan2f(R[4], R[0]);	//greek Y
			while(z > PI) z -= PI_2;
		    while(z < -PI) z += PI_2;
		return radianesagrados(z);
	}
	return 31337;
}

