/***************************************************************
/  Ifcaro3D - Preview                                          /
/                                                              /
/  Creado por Ifcaro <taslokooo@gmail.com>                     /
/  http://ps2dev.ifcaro.net                                    /
/                                                              /
/  Contiene codigo de ps2game:                                 /
/  Angel Garbayo Dominguez (neofar@ofcode.com)                 /
/  http://ps2dev.ofcode.com                                    /
/                                                              /
/  last mod: 20-11-2008                                        /
/  license:  GNU Library General Public License Version 2      /
/            (see LICENSE on the main directory)               /
***************************************************************/

#include "Ifcaro3D.h"

dWorldID world=0;
dSpaceID space=0;
dJointGroupID contactgroup=0;
dGeomID ground_box=0;

void nearCallback (void *data, dGeomID o1, dGeomID o2)   
{
  int i;
  //-- Obtener los cuerpos asociados
  dBodyID b1 = dGeomGetBody(o1);
  dBodyID b2 = dGeomGetBody(o2);

  //-- Si ya estan conectados por una articulacion, terminar
  if (b1 && b2 && dAreConnectedExcluding (b1,b2,dJointTypeContact)) return;
 
  //-- Crear los puntos de contacto, que en realidad son articulaciones
  //-- Para mas informacion consultar la documentacion de ODE
  //-- Aqui se puede especificar el tipo de superficies, los
  //-- coeficientes de rozamiento (mu), etc...
  dContact contact[MAX_CONTACTS]; 
  for (i=0; i<MAX_CONTACTS; i++) {
    contact[i].surface.mode = dContactBounce | dContactSoftCFM;
    contact[i].surface.mu = dInfinity;
    contact[i].surface.mu2 = 0;
    contact[i].surface.bounce = 0.1;
    contact[i].surface.bounce_vel = 0.1;
    contact[i].surface.soft_cfm = 0.01;
  }

  //-- Obtener los puntos de contacto
  int numc = dCollide (o1,o2,MAX_CONTACTS,&contact[0].geom, sizeof(dContact));

  //-- Si hay al menos algun punto de contacto...
  if (numc!=0) {
    
    //-- Para cada punto de contacto crear una articulacion
    for (i=0; i<numc; i++) {

      //-- Crear articulacion y meterla en el grupo contactgroup
      dJointID c = dJointCreateContact (world,contactgroup,&contact[i]);

      //-- Establecer la artiulacion entre los dos cuerpos
      dJointAttach (c,b1,b2);
    }
  }
}


void UpdateWorld(float veloc){
          dSpaceCollide (space,0,&nearCallback);
		  dWorldStep (world,veloc);
		  dJointGroupEmpty (contactgroup);
}
	
void ODE_crearmundo(float x, float y, float z){
  world = dWorldCreate();
  space = dHashSpaceCreate (0);
  contactgroup = dJointGroupCreate (0);
  dWorldSetGravity (world,x,y,z);
  dWorldSetCFM (world,1e-5);
  dWorldSetAutoDisableFlag (world,0);
  dWorldSetContactMaxCorrectingVel (world,0.1);
  dWorldSetContactSurfaceLayer (world,0.001);
  //dCreatePlane (space,0,0,1,0);
}

void SetWorldGravity(float x, float y, float z){

	if (world==0){
		ODE_crearmundo(x,y,z);
	}else{	
		dWorldSetGravity (world,x,y,z);
	}
}

void ODE_crearsuelo(float posx, float posy, float posz, float x, float y, float z){
  // environment
  ground_box = dCreateBox (space,x,y,z);
  dMatrix3 R;
  dRFromAxisAndAngle (R,0,1,0,0);
  dGeomSetPosition (ground_box,posx,posy,posz);
  dGeomSetRotation (ground_box,R);	
  }
  
void ODE_actualizarobjeto(Object3D *objeto, ODE_object obj){

	float x,y,z;
	
	x=dGeomGetPosition(obj.geom)[0];
	y=dGeomGetPosition(obj.geom)[1];
	z=dGeomGetPosition(obj.geom)[2];

	float QuatW=dGeomGetRotation(obj.geom)[0];
	float QuatX=dGeomGetRotation(obj.geom)[1];
	float QuatY=dGeomGetRotation(obj.geom)[2];
	float QuatZ=dGeomGetRotation(obj.geom)[3];

	float RotX = atan2f((2*(QuatY*QuatW))-(2*(QuatX*QuatZ)) , 1 - ((2*(QuatY*QuatY)) - (2*(QuatZ*QuatZ))));
	float RotY = sinf((2*(QuatX*QuatY)) + (2*(QuatZ*QuatW)));
	float RotZ = atan2f((2*(QuatX*QuatW))-(2*(QuatY*QuatZ)) , 1 - ((2*(QuatX*QuatX)) - (2*(QuatZ*QuatZ))));

	objectTranslate(objeto,0,0,0);
	
	objectRotate(objeto,RotX,RotY,RotZ);
	
	//printf("%d, %d, %d\n",(int)RotX,(int)RotY,(int)RotZ);
	
	// objectRotate(object,dGeomGetRotation(obj.geom)[0],dGeomGetRotation(obj.geom)[1],dGeomGetRotation(obj.geom)[2]);
	objectTranslate(objeto,x,y,z);//dGeomGetPosition(obj.geom)[0],dGeomGetPosition(obj.geom)[1],dGeomGetPosition(obj.geom)[2]);  
}

ODE_object ODE_creartrimesh(float posx, float posy, float posz, float x, float y, float z, Model3DM *model){
ODE_object obj;
  memset (&obj,0,sizeof(obj));
  dMass m;
  
  if (world==0) ODE_crearmundo(0,0,0);
  
  //-- Crear el Cuerpo y asociarlo al mundo
  obj.body = dBodyCreate (world);

  //-- Establecer la posicion inicial. Se pasan las coordenadas x,y,z
  //-- En este ejemplo el objeto esta en el origen, a una altura de 
  //-- 4 unidades
  dBodySetPosition(obj.body,-posx,posy,posz);

  //-- Establecer la rotacion
  //-- Cada objeto puede estar rotado. Se crea una matriz de
  //-- rotacion pasandole como parametros el vector que hace de
  //-- eje de giro y el angulo que se rota. En este ejemplo se pasa
  //-- el eje z y se rota '0' grados con respecto a el.
  //-- Probar este mismo ejemplo con los parametros (R,0,0,1,45)
  dMatrix3 R;
  dRFromAxisAndAngle(R,0,0,1,0);
  dBodySetRotation (obj.body,R);
  
  //-- Establecer la masa del cuerpo
  //-- Hay que especificar la masa total y las dimensiones
  //-- del cubo. Como masa se toma 0.5 y las dimensiones de la caja
  //-- son 0.5 x 0.5 x 0.1
  dMassSetBoxTotal (&m,1,x,y,z);
  dBodySetMass (obj.body,&m);

  //-- Crear la geometria: un cubo que se anadira al espacio para detectar
  //-- las colisiones. Las dimensiones son 0.5 x 0.5 x 0.1
  dTriMeshDataID Data = dGeomTriMeshDataCreate();
  
  dVector3 *Vertices=(dVector3*)malloc(sizeof(dVector3)*model->NUMVERTEX);
  int n;
  for (n=0;n<model->NUMVERTEX;n++){
	Vertices[n][0]=model->vertices[n][0];
	Vertices[n][1]=model->vertices[n][1];
	Vertices[n][2]=model->vertices[n][2];
  }  
  
  //printf("TRIMESH: Vertices %d, Caras %d\n",model->NUMVERTEX,model->NUMFACES);

	dGeomTriMeshDataBuildSimple(Data, (dReal*)Vertices, model->NUMVERTEX, model->faces, model->NUMFACES);

	obj.geom = dCreateTriMesh(space, Data, 0, 0, 0);



  //-- Asociar el cuerpo con la geometria
  dGeomSetBody (obj.geom,obj.body);
  return obj;
}

ODE_object ODE_crearcubo(float posx, float posy, float posz, float x, float y, float z){
ODE_object obj;
  memset (&obj,0,sizeof(obj));
  dMass m;
  
  if (world==0) ODE_crearmundo(0,0,0);
  
  //-- Crear el Cuerpo y asociarlo al mundo
  obj.body = dBodyCreate (world);

  //-- Establecer la posicion inicial. Se pasan las coordenadas x,y,z
  //-- En este ejemplo el objeto esta en el origen, a una altura de 
  //-- 4 unidades
  dBodySetPosition(obj.body,-posx,posy,posz);

  //-- Establecer la rotacion
  //-- Cada objeto puede estar rotado. Se crea una matriz de
  //-- rotacion pasandole como parametros el vector que hace de
  //-- eje de giro y el angulo que se rota. En este ejemplo se pasa
  //-- el eje z y se rota '0' grados con respecto a el.
  //-- Probar este mismo ejemplo con los parametros (R,0,0,1,45)
  dMatrix3 R;
  dRFromAxisAndAngle(R,0,0,1,0);
  dBodySetRotation (obj.body,R);
  
  //-- Establecer la masa del cuerpo
  //-- Hay que especificar la masa total y las dimensiones
  //-- del cubo. Como masa se toma 0.5 y las dimensiones de la caja
  //-- son 0.5 x 0.5 x 0.1
  dMassSetBoxTotal (&m,1,x,y,z);
  dBodySetMass (obj.body,&m);

  //-- Crear la geometria: un cubo que se anadira al espacio para detectar
  //-- las colisiones. Las dimensiones son 0.5 x 0.5 x 0.1
  obj.geom= malloc(sizeof(dGeomID));
  obj.geom = dCreateBox (space,x+5,y+5,z+5);

  //-- Asociar el cuerpo con la geometria
  dGeomSetBody (obj.geom,obj.body);
  return obj;
}

ODE_object ODE_crearesfera(float posx, float posy, float posz, float radio){
ODE_object obj;
  memset (&obj,0,sizeof(obj));
  dMass m;
  
  if (world==0) ODE_crearmundo(0,0,0);
  
  //-- Crear el Cuerpo y asociarlo al mundo
  obj.body = dBodyCreate (world);

  //-- Establecer la posicion inicial. Se pasan las coordenadas x,y,z
  //-- En este ejemplo el objeto esta en el origen, a una altura de 
  //-- 4 unidades
  dBodySetPosition(obj.body,-posx,posy,posz);

  //-- Establecer la rotacion
  //-- Cada objeto puede estar rotado. Se crea una matriz de
  //-- rotacion pasandole como parametros el vector que hace de
  //-- eje de giro y el angulo que se rota. En este ejemplo se pasa
  //-- el eje z y se rota '0' grados con respecto a el.
  //-- Probar este mismo ejemplo con los parametros (R,0,0,1,45)
  dMatrix3 R;
  dRFromAxisAndAngle(R,0,0,1,0);
  dBodySetRotation (obj.body,R);
  
  //-- Establecer la masa del cuerpo
  //-- Hay que especificar la masa total y las dimensiones
  //-- del cubo. Como masa se toma 0.5 y las dimensiones de la caja
  //-- son 0.5 x 0.5 x 0.1
  dMassSetBoxTotal (&m,1,1,1,1);
  dBodySetMass (obj.body,&m);

  //-- Crear la geometria: un cubo que se anadira al espacio para detectar
  //-- las colisiones. Las dimensiones son 0.5 x 0.5 x 0.1
  obj.geom= malloc(sizeof(dGeomID));
  //obj.geom = dCreateBox (space,x+5,y+5,z+5);
  obj.geom = dCreateSphere(space,radio);
  //-- Asociar el cuerpo con la geometria
  dGeomSetBody (obj.geom,obj.body);
  return obj;
}

void ODE_posicionobjeto(ODE_object obj, float x, float y, float z){
	dBodySetPosition(obj.body,-x,y,z);
	//objectTranslate(objeto,x,y,z);
}

void ODE_moverobjeto(ODE_object obj, float x, float y, float z){
	dBodySetPosition(obj.body,dBodyGetPosition(obj.body)[0]-x,dBodyGetPosition(obj.body)[1]+y,dBodyGetPosition(obj.body)[2]+z);
	//objectTranslate(objeto,x,y,z);
}

void ODE_rotarobjeto(ODE_object obj, float x, float y, float z){

/*libVu0FMatrix m;
libVu0UnitMatrix(m);

libVu0FVector rot;

	rot[0] = -x;
    rot[1] = -y;
    rot[2] = -z;
	
    // cuidado con pasarnos por encima
    while(rot[0] > PI) rot[0] -= PI_2;
    while(rot[1] > PI) rot[1] -= PI_2;
    while(rot[2] > PI) rot[2] -= PI_2;

    // y por debajo
    while(rot[0] < -PI) rot[0] += PI_2;
    while(rot[1] < -PI) rot[1] += PI_2;
    while(rot[2] < -PI) rot[2] += PI_2;
  
  libVu0RotMatrix(m, m, rot);	
  
  dBodySetRotation(obj.body,m[0]);*/
  
  
   dMatrix3 R;
  dRFromEulerAngles(R, x, y, z);
dBodySetRotation(obj.body,R);

}		

void ODE_girarobjeto(ODE_object obj, float x, float y, float z){

dMatrix3 R;
dMatrix3 R1;
dMatrix3 *R2;
  dRFromEulerAngles(R1, x, y, z);
  R2=(dMatrix3*)dBodyGetRotation(obj.body);
  dMULTIPLY0_333(R, R1, R2[0]); 
dBodySetRotation(obj.body,R);

}		
