/***************************************************************
/  Ifcaro3D - Preview                                          /
/                                                              /
/  Creado por Ifcaro <taslokooo@gmail.com>                     /
/  http://ps2dev.ifcaro.net                                    /
/                                                              /
/  Contiene codigo de ps2game:                                 /
/  Angel Garbayo Dominguez (neofar@ofcode.com)                 /
/  http://ps2dev.ofcode.com                                    /
/                                                              /
/  last mod: 20-11-2008                                        /
/  license:  GNU Library General Public License Version 2      /
/            (see LICENSE on the main directory)               /
***************************************************************/

#include "Ifcaro3D.h"

#define MAX_GAME_STATUS 64
#define GAMESTATUS_VOID -1

typedef struct
	{
	int id;
	void (*ProcStart)();  	
	void (*ProcTick)();  	
	void (*ProcStop)();  	
	} GameStatusBlock;


GameStatusBlock status[MAX_GAME_STATUS];
int GameStatusLevel    = GAMESTATUS_VOID;
int GameStatusNewLevel = GAMESTATUS_VOID;


void SysGameStart()
	{
	int i;
	// ponemos el status a vacio
    GameStatusLevel    = GAMESTATUS_VOID;
    GameStatusNewLevel = GAMESTATUS_VOID;
    
    // reiniciamos la pila
    for(i=0;i<MAX_GAME_STATUS;i++)
    	{
   		status[i].id = GAMESTATUS_VOID;
   		status[i].ProcStart = NULL;
   		status[i].ProcTick  = NULL;
   		status[i].ProcStop  = NULL;
    	}    
	}    

int SysGameGetFreeID()
	{
	int i;

    for(i=0;i<MAX_GAME_STATUS;i++)
   		if (status[i].id==GAMESTATUS_VOID) return i;

    return -1;
	}

void SysGameRegister(int id, void (*ProcStart)(), void (*ProcTick)(), void (*ProcStop)())
	{
	// Buscamos un ID LIbre
	int index = SysGameGetFreeID();
	printf("SysGameRegister - id = %d \n", index);
	
	if(index ==-1)
		printf("ERROR: SysGameRegister - sin espacio de entorno");
	else
		{
 		status[index].id = id;
   		status[index].ProcStart = ProcStart;
   		status[index].ProcTick  = ProcTick;
   		status[index].ProcStop  = ProcStop;
   		}  			    
	}    
	
int SysGameGetID(int id)
	{
	int i;

	// Buscamos en la pila este id
    for(i=0;i<MAX_GAME_STATUS;i++)
   		if (id == status[i].id )
          return i;

    return -1;
	}    

int SysGameStatus()
    {
    return SysGameGetID(GameStatusLevel);
    }

int SysGameIsStatus(int modo)
    {
    return (SysGameStatus() == modo);
    }

// aqui es donde realmente hacemos el cambio
// miramos en que estado nos encotrarmos y llamamos a su STOP
// cambiamos de estado y llamamos al START
void SysGameProcessChange()
    {

    // si no hay nuevo estado, pos no cambiamos
    if(GameStatusNewLevel == GAMESTATUS_VOID) return;
    
    // Llamamos al metodo STOP del estado actual
    if(GameStatusLevel!=GAMESTATUS_VOID)
        {
        int id = SysGameGetID(GameStatusLevel);
        if(id!=-1)
            {
        	if(status[id].ProcStop != NULL)
        		status[id].ProcStop();
            }
        }
    // Pasamos al nuevo status
    GameStatusLevel = GameStatusNewLevel;
    GameStatusNewLevel = GAMESTATUS_VOID;


    // Llamamos al START del Nuevo estado
    if(GameStatusLevel!=GAMESTATUS_VOID)
        {
        int id = SysGameGetID(GameStatusLevel);
        if(id!=-1)
            {
        	if(status[id].ProcStart != NULL)
        		status[id].ProcStart();
            }
        }
        
    }


void SysGameChangeTo(int mode)
    {

    // si ya estamos en este estado no hacemos nada
    if (GameStatusLevel == mode) return;

    // marcamos el estado en el que queremos entrar
    GameStatusNewLevel = mode;
    
    // si no nos encontramos en ningun estado podemos cambiar ya
    if (GameStatusLevel == GAMESTATUS_VOID)
        SysGameProcessChange();
    }
    

void SysGameTick()
	{
	 // comprobamos si es necesario cambiar de estado
 	SysGameProcessChange();

 	// si no estamos en ningun estado salimos
    if (SysGameIsStatus(GAMESTATUS_VOID))
        return;

    // Llamamos al metodo Tick
    int id = SysGameGetID(GameStatusLevel);
    if(id!=-1)
        {
        if(status[id].ProcTick != NULL)
        	status[id].ProcTick();
       	}

	}    

